"""Deploy resource for deployment-related endpoints."""

from ..models.deploy import (
    Deploy as DeployModel,
    DeployActionResponse,
    DeploysResponse,
    DeploySymphoniesResponse,
    MarketHoursResponse,
)


class DeployResource:
    """Resource for deploy-related endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def get_market_hours(self) -> MarketHoursResponse:
        """
        Get market hours schedule for the upcoming week.

        Returns
        -------
            Market hours schedule including open/close times for each day
        """
        response = self._client.get("/api/v1/deploy/market-hours")
        return MarketHoursResponse.model_validate(response)

    def get_deploy_symphonies(
        self,
        account_id: str,
        asset_class: str | None = None,
    ) -> DeploySymphoniesResponse:
        """
        Get metadata for symphonies which have new invests pending.

        Args:
            account_id: Unique identifier (UUID) of the account
            asset_class: Deprecated.

        Returns
        -------
            List of symphonies with pending invests
        """
        params = {}
        if asset_class:
            params["asset_class"] = asset_class
        response = self._client.get(
            f"/api/v1/deploy/accounts/{account_id}/deploy-symphonies",
            params=params if params else None,
        )
        return DeploySymphoniesResponse.model_validate(response)

    def get_deploys(
        self,
        account_id: str,
        status: str | None = None,
        limit: int | None = None,
    ) -> DeploysResponse:
        """
        Get all deploy details for an account.

        Args:
            account_id: Unique identifier (UUID) of the account
            status: Filter by status (QUEUED, CANCELED, SUCCEEDED, FAILED, REJECTED, EXPIRED)
            limit: Maximum number of results

        Returns
        -------
            List of deploys for the account
        """
        params = {}
        if status:
            params["status"] = status
        if limit is not None:
            params["limit"] = limit
        response = self._client.get(
            f"/api/v1/deploy/accounts/{account_id}/deploys",
            params=params if params else None,
        )
        return DeploysResponse.model_validate(response)

    def get_deploy(
        self,
        account_id: str,
        deploy_id: str,
    ) -> DeployModel:
        """
        Get a specific deploy's details.

        Args:
            account_id: Unique identifier (UUID) of the account
            deploy_id: Unique identifier for the deployment operation

        Returns
        -------
            Details of the specified deploy
        """
        response = self._client.get(f"/api/v1/deploy/accounts/{account_id}/deploys/{deploy_id}")
        return DeployModel.model_validate(response)

    def delete_deploy(
        self,
        account_id: str,
        deploy_id: str,
    ) -> None:
        """
        Cancel a pending deploy.

        Args:
            account_id: Unique identifier (UUID) of the account
            deploy_id: Unique identifier for the deployment operation
        """
        self._client.delete(f"/api/v1/deploy/accounts/{account_id}/deploys/{deploy_id}")

    def go_to_cash(
        self,
        account_id: str,
        symphony_id: str,
    ) -> DeployActionResponse:
        """
        Sell all assets in a symphony, leaving proceeds in cash.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony

        Returns
        -------
            Information about the deploy
        """
        response = self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/go-to-cash"
        )
        return DeployActionResponse.model_validate(response)

    def invest(
        self,
        account_id: str,
        symphony_id: str,
        amount: float,
    ) -> DeployActionResponse:
        """
        Invest cash into a Symphony.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony
            amount: Amount of cash to invest (in USD)

        Returns
        -------
            Information about the deploy
        """
        response = self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/invest",
            json={"amount": amount},
        )
        return DeployActionResponse.model_validate(response)

    def liquidate(
        self,
        account_id: str,
        symphony_id: str,
    ) -> DeployActionResponse:
        """
        Liquidate a symphony entirely.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony

        Returns
        -------
            Information about the deploy
        """
        response = self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/liquidate"
        )
        return DeployActionResponse.model_validate(response)

    def rebalance(
        self,
        account_id: str,
        symphony_id: str,
        rebalance_request_uuid: str,
    ) -> DeployActionResponse:
        """
        Rebalance a symphony NOW.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony
            rebalance_request_uuid: UUID for the rebalance request

        Returns
        -------
            Information about the deploy
        """
        response = self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/rebalance",
            json={"rebalance_request_uuid": rebalance_request_uuid},
        )
        return DeployActionResponse.model_validate(response)

    def skip_automated_rebalance(
        self,
        account_id: str,
        symphony_id: str,
        skip: bool,
    ) -> None:
        """
        Skip automated rebalancing.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony
            skip: Whether to skip the next automated rebalance
        """
        self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/skip-automated-rebalance",
            json={"skip": skip},
        )

    def withdraw(
        self,
        account_id: str,
        symphony_id: str,
        amount: float,
    ) -> DeployActionResponse:
        """
        Withdraw cash from a Symphony.

        Args:
            account_id: Unique identifier (UUID) of the account
            symphony_id: Unique identifier for the Symphony
            amount: Amount of cash to withdraw (in USD)

        Returns
        -------
            Information about the deploy
        """
        response = self._client.post(
            f"/api/v1/deploy/accounts/{account_id}/symphonies/{symphony_id}/withdraw",
            json={"amount": amount},
        )
        return DeployActionResponse.model_validate(response)



"""Backtest resource for running backtest simulations."""

from typing import Any

from ..models.backtest import (
    ApplySubscription,
    BacktestRequest,
    BacktestResult,
    BacktestVersion,
    Broker,
    ConfigEntry,
    ConfigKey,
    RebalanceRequest,
    RebalanceResult,
)
from ..models.common import SymphonyDefinition


class Backtest:
    """Backtest resource for running backtest simulations.

    This resource provides methods for running backtests, rebalances,
    and managing backtest configurations.
    """

    def __init__(self, http_client):
        self.http_client = http_client

    def run(self, request: BacktestRequest | dict[str, Any]) -> BacktestResult:
        """
        Run a generic backtest simulation (v1).

        Args:
            request: BacktestRequest model or dictionary matching the schema.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics

        Example:
            # Backtest with full symphony definition
            result = client.backtest.run(
                BacktestRequest(
                    symphony=SymphonyDefinition(
                        name="My Strategy",
                        rebalance="daily",
                        children=[...]
                    )
                )
            )
        """
        payload = request
        if isinstance(request, BacktestRequest):
            payload = request.model_dump(by_alias=True, exclude_none=True, mode="json")
            if "symphony" in payload and payload["symphony"]:
                payload["symphony"] = {"raw_value": payload["symphony"]}

        raw_response = self.http_client.post("/api/v1/backtest", json=payload)
        return BacktestResult.model_validate(raw_response)

    def run_v2(
        self,
        symphony: SymphonyDefinition | dict[str, Any] | None = None,
        capital: float = 10000.0,
        abbreviate_days: int | None = None,
        apply_reg_fee: bool = True,
        apply_taf_fee: bool = True,
        apply_subscription: ApplySubscription = ApplySubscription.NONE,
        backtest_version: BacktestVersion = BacktestVersion.V2,
        slippage_percent: float = 0.0001,
        spread_markup: float = 0.0,
        start_date: str | None = None,
        end_date: str | None = None,
        broker: Broker = Broker.ALPACA_WHITE_LABEL,
        benchmark_symphonies: list[str] | None = None,
        benchmark_tickers: list[str] | None = None,
        sparkgraph_color: str | None = None,
    ) -> BacktestResult:
        """
        Run a generic backtest simulation (v2).

        Args:
            symphony: Symphony definition to backtest (SymphonyDefinition model or dict).
            capital: Initial capital for the backtest (default: 10000.0).
            abbreviate_days: Number of days to abbreviate the backtest (for testing).
            apply_reg_fee: Whether to apply regulatory fees (SEC fees).
            apply_taf_fee: Whether to apply TAF (Trading Activity Fee).
            apply_subscription: Composer subscription level to simulate (affects fees).
            backtest_version: Backtest engine version to use.
            slippage_percent: Slippage assumption as decimal (0.0001 = 0.01%).
            spread_markup: Bid-ask spread markup as decimal (0.001 = 0.1%).
            start_date: Backtest start date (YYYY-MM-DD). Defaults to earliest available data.
            end_date: Backtest end date (YYYY-MM-DD). Defaults to latest available data.
            broker: Broker to simulate for fee calculations.
            benchmark_symphonies: List of symphony IDs to use as benchmarks.
            benchmark_tickers: List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ']).
            sparkgraph_color: Custom color for performance chart.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics

        Example:
             result = client.backtest.run_v2(symphony=SymphonyDefinition(...))
             print(f"Sharpe: {result.stats.sharpe_ratio}")

        Example with dict:
             score_data = {"step": "root", "name": "My Strategy", ...}
             result = client.backtest.run_v2(symphony=score_data)
        """
        if symphony is not None:
            if isinstance(symphony, dict):
                raw_value = symphony
            else:
                raw_value = symphony.model_dump(by_alias=True, exclude_none=True)
            symphony_payload = {"raw_value": raw_value}
        else:
            symphony_payload = None

        payload = {
            "symphony": symphony_payload,
            "capital": capital,
            "abbreviate_days": abbreviate_days,
            "apply_reg_fee": apply_reg_fee,
            "apply_taf_fee": apply_taf_fee,
            "apply_subscription": apply_subscription.value if apply_subscription else None,
            "backtest_version": backtest_version.value if backtest_version else None,
            "slippage_percent": slippage_percent,
            "spread_markup": spread_markup,
            "start_date": start_date,
            "end_date": end_date,
            "broker": broker.value if broker else None,
            "benchmark_symphonies": benchmark_symphonies,
            "benchmark_tickers": benchmark_tickers,
            "sparkgraph_color": sparkgraph_color,
        }
        payload = {k: v for k, v in payload.items() if v is not None}

        raw_response = self.http_client.post("/api/v2/backtest", json=payload)
        return BacktestResult.model_validate(raw_response)

    def run_public_v2(
        self,
        symphony: SymphonyDefinition | dict[str, Any] | None = None,
        capital: float = 10000.0,
        abbreviate_days: int | None = None,
        apply_reg_fee: bool = True,
        apply_taf_fee: bool = True,
        apply_subscription: ApplySubscription = ApplySubscription.NONE,
        backtest_version: BacktestVersion = BacktestVersion.V2,
        slippage_percent: float = 0.0001,
        spread_markup: float = 0.0,
        start_date: str | None = None,
        end_date: str | None = None,
        broker: Broker = Broker.ALPACA_WHITE_LABEL,
        benchmark_symphonies: list[str] | None = None,
        benchmark_tickers: list[str] | None = None,
        sparkgraph_color: str | None = None,
    ) -> BacktestResult:
        """
        Run a public backtest simulation (v2).

        Args:
            symphony: Symphony definition to backtest (SymphonyDefinition model or dict).
            capital: Initial capital for the backtest (default: 10000.0).
            abbreviate_days: Number of days to abbreviate the backtest (for testing).
            apply_reg_fee: Whether to apply regulatory fees (SEC fees).
            apply_taf_fee: Whether to apply TAF (Trading Activity Fee).
            apply_subscription: Composer subscription level to simulate (affects fees).
            backtest_version: Backtest engine version to use.
            slippage_percent: Slippage assumption as decimal (0.0001 = 0.01%).
            spread_markup: Bid-ask spread markup as decimal (0.001 = 0.1%).
            start_date: Backtest start date (YYYY-MM-DD). Defaults to earliest available data.
            end_date: Backtest end date (YYYY-MM-DD). Defaults to latest available data.
            broker: Broker to simulate for fee calculations.
            benchmark_symphonies: List of symphony IDs to use as benchmarks.
            benchmark_tickers: List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ']).
            sparkgraph_color: Custom color for performance chart.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics

        Example:
             result = client.backtest.run_public_v2(symphony=SymphonyDefinition(...))
             print(f"Sharpe: {result.stats.sharpe_ratio}")

        Example with dict:
             score_data = {"step": "root", "name": "My Strategy", ...}
             result = client.backtest.run_public_v2(symphony=score_data)
        """
        if symphony is not None:
            if isinstance(symphony, dict):
                raw_value = symphony
            else:
                raw_value = symphony.model_dump(by_alias=True, exclude_none=True)
            symphony_payload = {"raw_value": raw_value}
        else:
            symphony_payload = None

        payload = {
            "symphony": symphony_payload,
            "capital": capital,
            "abbreviate_days": abbreviate_days,
            "apply_reg_fee": apply_reg_fee,
            "apply_taf_fee": apply_taf_fee,
            "apply_subscription": apply_subscription.value if apply_subscription else None,
            "backtest_version": backtest_version.value if backtest_version else None,
            "slippage_percent": slippage_percent,
            "spread_markup": spread_markup,
            "start_date": start_date,
            "end_date": end_date,
            "broker": broker.value if broker else None,
            "benchmark_symphonies": benchmark_symphonies,
            "benchmark_tickers": benchmark_tickers,
            "sparkgraph_color": sparkgraph_color,
        }
        payload = {k: v for k, v in payload.items() if v is not None}

        raw_response = self.http_client.post("/api/v2/public/backtest", json=payload)
        return BacktestResult.model_validate(raw_response)

    def rebalance(self, request: RebalanceRequest | dict[str, Any]) -> RebalanceResult:
        """
        Run a rebalance for specified symphonies.

        Args:
            request: RebalanceRequest model or dictionary matching the schema.

        Returns
        -------
            RebalanceResult: Rebalance result with quotes and run results.

        Example:
             request = RebalanceRequest(
            ...     symphonies={
            ...         "sym-123": SymphonyRebalanceState(cash=10000, shares={})
            ...     }
            ... )
             result = client.backtest.rebalance(request)
             print(result.run_results)
        """
        payload = request
        if isinstance(request, RebalanceRequest):
            payload = request.model_dump(by_alias=True, exclude_none=True, mode="json")

        raw_response = self.http_client.post("/api/v2/rebalance", json=payload)
        return RebalanceResult.model_validate(raw_response)

    def list_configs(self, keys: list[str] | None = None) -> list[ConfigEntry]:
        """
        List accessible config entries.

        Args:
            keys: Optional list of config keys to filter by.
                Valid keys: "constants", "deposit_presets_config", "openai-prompt",
                "openai_config"

        Returns
        -------
            List of ConfigEntry objects

        Example:
            configs = client.backtest.list_configs()
            for cfg in configs:
                print(f"{cfg.config_key}: {cfg.config}")
        """
        params = {"keys": keys} if keys else None
        raw_response = self.http_client.get("/api/v1/configs", params=params)
        return [ConfigEntry.from_dict(c) for c in raw_response]

    def get_config(self, config_key: ConfigKey | str) -> ConfigEntry:
        """
        Get a single config entry by key.

        Args:
            config_key: The config key to retrieve.
                Valid keys: "constants", "deposit_presets_config", "openai-prompt",
                "openai_config"
                Can pass ConfigKey enum or string.

        Returns
        -------
            ConfigEntry object

        Example:
            config = client.backtest.get_config("openai_config")
            print(config.config_key)
            print(config.config.system_prompt)
        """
        key_value = config_key.value if isinstance(config_key, ConfigKey) else config_key
        raw_response = self.http_client.get(f"/api/v1/configs/{key_value}")
        return ConfigEntry.from_dict(raw_response)



"""Portfolio models - request and response models for portfolio endpoints."""

from .responses import (
    ActivityHistoryItem,
    ActivityHistoryResponse,
    AssetClass,
    DeployHistoryItem,
    DeployHistoryResponse,
    HoldingAllocation,
    HoldingStats,
    HoldingStatsResponse,
    OptionsDetails,
    PortfolioHistory,
    PositionDirection,
    SymphonyAllocation,
    SymphonyHolding,
    SymphonyHoldingAllocation,
    SymphonyHoldings,
    SymphonyStats,
    SymphonyStatsMeta,
    SymphonyStatsMetaResponse,
    SymphonyStatsResponse,
    TimeSeries,
    TotalStats,
)

__all__ = [
    "AssetClass",
    "PositionDirection",
    "OptionsDetails",
    "HoldingAllocation",
    "SymphonyAllocation",
    "SymphonyHoldingAllocation",
    "HoldingStats",
    "HoldingStatsResponse",
    "TotalStats",
    "SymphonyHolding",
    "SymphonyStats",
    "SymphonyStatsMeta",
    "SymphonyStatsMetaResponse",
    "SymphonyStatsResponse",
    "TimeSeries",
    "PortfolioHistory",
    "SymphonyHoldings",
    "ActivityHistoryItem",
    "ActivityHistoryResponse",
    "DeployHistoryItem",
    "DeployHistoryResponse",
]



"""Public Symphony resource - public endpoints for accessing symphony data."""

from typing import Any

from ..http_client import HTTPClient
from ..models.backtest import (
    ApplySubscription,
    BacktestResult,
    BacktestVersion,
    Broker,
    Indicator,
    SymphonyDetail,
    SymphonyMeta,
    SymphonyMetaResponse,
    SymphonyVersionInfo,
    TickersResponse,
)
from ..models.common import SymphonyDefinition


class PublicSymphony:
    """
    Public symphony endpoints (no authentication required).

    These endpoints provide read-only access to publicly shared symphonies
    and technical indicators.
    """

    def __init__(self, http_client: HTTPClient):
        self._client = http_client

    def get_indicators(self) -> list[Indicator]:
        """
        Get a list of all available technical indicators.

        Returns a list of indicators that can be used in symphony conditions,
        including moving averages, RSI, standard deviation, etc.

        Returns
        -------
            List[Indicator]: List of available technical indicators with their
                parameters and metadata.

        Example:
             indicators = client.public_symphony.get_indicators()
             for indicator in indicators:
            ...     print(f"{indicator.name}: {indicator.description}")
        """
        response = self._client.get("/api/v1/public/symphony-scores/indicators")
        return [Indicator.model_validate(item) for item in response]

    def get_symphony_meta(self, symphony_ids: list[str]) -> list[SymphonyMeta]:
        """
        Get metadata for symphonies by their IDs.

        Args:
            symphony_ids: List of symphony IDs to look up.

        Returns
        -------
            List[SymphonyMeta]: List of symphony metadata objects.

        Example:
             meta = client.public_symphony.get_symphony_meta(["sym-abc123"])
             for symphony in meta:
            ...     print(f"{symphony.name}: {symphony.rebalance_frequency}")
        """
        response = self._client.post(
            "/api/v1/public/meta/symphonies", json={"symphony_id": symphony_ids}
        )
        result = SymphonyMetaResponse.model_validate(response)
        return result.symphonies

    def get_symphony(self, symphony_id: str) -> SymphonyDetail:
        """
        Get detailed information about a symphony.

        Args:
            symphony_id: Unique identifier for the symphony.

        Returns
        -------
            SymphonyDetail: Detailed symphony information including stats,
                backtest data, and metadata.

        Example:
             symphony = client.public_symphony.get_symphony("sym-abc123")
             print(f"Name: {symphony.name}")
             print(f"Sharpe: {symphony.stats_oos_sharpe_ratio}")
        """
        response = self._client.get(f"/api/v1/public/symphonies/{symphony_id}")
        return SymphonyDetail.model_validate(response)

    def get_versions(self, symphony_id: str) -> list[SymphonyVersionInfo]:
        """
        Get all versions for a symphony.

        Args:
            symphony_id: Unique identifier for the symphony.

        Returns
        -------
            List[SymphonyVersionInfo]: List of version information.

        Example:
             versions = client.public_symphony.get_versions("sym-abc123")
             for version in versions:
            ...     print(f"Version {version.version_id}: {version.created_at}")
        """
        response = self._client.get(f"/api/v1/public/symphonies/{symphony_id}/versions")
        return [SymphonyVersionInfo.model_validate(item) for item in response]

    def get_version_score(
        self, symphony_id: str, version_id: str, score_version: str = "v1"
    ) -> SymphonyDefinition:
        """
        Get an existing symphony version's EDN (score).

        Args:
            symphony_id: Unique identifier for the symphony.
            version_id: Unique identifier for the symphony version.
            score_version: Score version to retrieve ("v1" or "v2"). Defaults to "v1".

        Returns
        -------
            SymphonyDefinition: The symphony score/parsed EDN structure.

        Example:
             score = client.public_symphony.get_version_score(
            ...     "sym-abc123",
            ...     "ver-xyz789"
            ... )
             print(score.name)
             print(score.rebalance)
        """
        params = {"score_version": score_version}
        response = self._client.get(
            f"/api/v1/public/symphonies/{symphony_id}/versions/{version_id}/score",
            params=params,
        )
        return SymphonyDefinition.model_validate(response)

    def get_score(self, symphony_id: str, score_version: str = "v1") -> SymphonyDefinition:
        """
        Get an existing symphony's EDN (score).

        Args:
            symphony_id: Unique identifier for the symphony.
            score_version: Score version to retrieve ("v1" or "v2"). Defaults to "v1".

        Returns
        -------
            SymphonyDefinition: The symphony score/parsed EDN structure.

        Example:
             score = client.public_symphony.get_score("sym-abc123")
             print(score.name)
             print(score.rebalance)
        """
        params = {"score_version": score_version}
        response = self._client.get(f"/api/v1/public/symphonies/{symphony_id}/score", params=params)
        return SymphonyDefinition.model_validate(response)

    def get_tickers(self, symphony_id: str) -> list[str]:
        """
        Get all tickers used in a symphony.

        Args:
            symphony_id: Unique identifier for the symphony.

        Returns
        -------
            List[str]: List of ticker symbols used in the symphony.

        Example:
             tickers = client.public_symphony.get_tickers("sym-abc123")
             print(f"Symphony uses: {', '.join(tickers)}")
        """
        response = self._client.get(f"/api/v1/public/symphonies/{symphony_id}/tickers")
        result = TickersResponse.model_validate(response)
        return result.tickers

    def backtest_symphony(
        self,
        symphony_id: str,
        capital: float = 10000.0,
        abbreviate_days: int | None = None,
        apply_reg_fee: bool = True,
        apply_taf_fee: bool = True,
        apply_subscription: ApplySubscription = ApplySubscription.NONE,
        backtest_version: BacktestVersion = BacktestVersion.V2,
        slippage_percent: float = 0.0001,
        spread_markup: float = 0.0,
        start_date: str | None = None,
        end_date: str | None = None,
        broker: Broker = Broker.ALPACA_WHITE_LABEL,
        benchmark_symphonies: list[str] | None = None,
        benchmark_tickers: list[str] | None = None,
        sparkgraph_color: str | None = None,
    ) -> BacktestResult:
        """
        Run a backtest for a public symphony by its ID.

        Args:
            symphony_id: The ID of the symphony to backtest.
            capital: Initial capital for the backtest (default: 10000.0).
            abbreviate_days: Number of days to abbreviate the backtest (for testing).
            apply_reg_fee: Whether to apply regulatory fees (SEC fees).
            apply_taf_fee: Whether to apply TAF (Trading Activity Fee).
            apply_subscription: Composer subscription level to simulate (affects fees).
            backtest_version: Backtest engine version to use.
            slippage_percent: Slippage assumption as decimal (0.0001 = 0.01%).
            spread_markup: Bid-ask spread markup as decimal (0.001 = 0.1%).
            start_date: Backtest start date (YYYY-MM-DD). Defaults to earliest available data.
            end_date: Backtest end date (YYYY-MM-DD). Defaults to latest available data.
            broker: Broker to simulate for fee calculations.
            benchmark_symphonies: List of symphony IDs to use as benchmarks.
            benchmark_tickers: List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ']).
            sparkgraph_color: Custom color for performance chart.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics.

        Example:
            result = client.public_symphony.backtest_symphony("sym-abc123")
            print(f"Sharpe: {result.stats.sharpe_ratio}")
        """
        payload = {
            "capital": capital,
            "abbreviate_days": abbreviate_days,
            "apply_reg_fee": apply_reg_fee,
            "apply_taf_fee": apply_taf_fee,
            "apply_subscription": apply_subscription.value if apply_subscription else None,
            "backtest_version": backtest_version.value if backtest_version else None,
            "slippage_percent": slippage_percent,
            "spread_markup": spread_markup,
            "start_date": start_date,
            "end_date": end_date,
            "broker": broker.value if broker else None,
            "benchmark_symphonies": benchmark_symphonies,
            "benchmark_tickers": benchmark_tickers,
            "sparkgraph_color": sparkgraph_color,
        }
        payload = {k: v for k, v in payload.items() if v is not None}

        raw_response = self._client.post(
            f"/api/v1/public/symphonies/{symphony_id}/backtest",
            json=payload,
        )
        return BacktestResult.model_validate(raw_response)

    def backtest(
        self,
        symphony: SymphonyDefinition | dict[str, Any],
        capital: float = 10000.0,
        abbreviate_days: int | None = None,
        apply_reg_fee: bool = True,
        apply_taf_fee: bool = True,
        apply_subscription: ApplySubscription = ApplySubscription.NONE,
        backtest_version: BacktestVersion = BacktestVersion.V2,
        slippage_percent: float = 0.0001,
        spread_markup: float = 0.0,
        start_date: str | None = None,
        end_date: str | None = None,
        broker: Broker = Broker.ALPACA_WHITE_LABEL,
        benchmark_symphonies: list[str] | None = None,
        benchmark_tickers: list[str] | None = None,
        sparkgraph_color: str | None = None,
    ) -> BacktestResult:
        """
        Run a standalone backtest with a custom symphony definition.

        Args:
            symphony: Symphony definition to backtest (SymphonyDefinition model or dict).
            capital: Initial capital for the backtest (default: 10000.0).
            abbreviate_days: Number of days to abbreviate the backtest (for testing).
            apply_reg_fee: Whether to apply regulatory fees (SEC fees).
            apply_taf_fee: Whether to apply TAF (Trading Activity Fee).
            apply_subscription: Composer subscription level to simulate (affects fees).
            backtest_version: Backtest engine version to use.
            slippage_percent: Slippage assumption as decimal (0.0001 = 0.01%).
            spread_markup: Bid-ask spread markup as decimal (0.001 = 0.1%).
            start_date: Backtest start date (YYYY-MM-DD). Defaults to earliest available data.
            end_date: Backtest end date (YYYY-MM-DD). Defaults to latest available data.
            broker: Broker to simulate for fee calculations.
            benchmark_symphonies: List of symphony IDs to use as benchmarks.
            benchmark_tickers: List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ']).
            sparkgraph_color: Custom color for performance chart.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics.

        Example:
             result = client.public_symphony.backtest(symphony=SymphonyDefinition(...))
             print(f"Sharpe: {result.stats.sharpe_ratio}")

        Example with dict:
             score_data = {"step": "root", "name": "My Strategy", ...}
             result = client.public_symphony.backtest(symphony=score_data)
        """
        if isinstance(symphony, dict):
            raw_value = symphony
        else:
            raw_value = symphony.model_dump(by_alias=True, exclude_none=True)

        payload = {
            "symphony": {"raw_value": raw_value},
            "capital": capital,
            "abbreviate_days": abbreviate_days,
            "apply_reg_fee": apply_reg_fee,
            "apply_taf_fee": apply_taf_fee,
            "apply_subscription": apply_subscription.value if apply_subscription else None,
            "backtest_version": backtest_version.value if backtest_version else None,
            "slippage_percent": slippage_percent,
            "spread_markup": spread_markup,
            "start_date": start_date,
            "end_date": end_date,
            "broker": broker.value if broker else None,
            "benchmark_symphonies": benchmark_symphonies,
            "benchmark_tickers": benchmark_tickers,
            "sparkgraph_color": sparkgraph_color,
        }
        payload = {k: v for k, v in payload.items() if v is not None}

        raw_response = self._client.post(
            "/api/v1/public/backtest",
            json=payload,
        )
        return BacktestResult.model_validate(raw_response)

    def backtest_symphony_v2(
        self,
        symphony_id: str,
        capital: float = 10000.0,
        abbreviate_days: int | None = None,
        apply_reg_fee: bool = True,
        apply_taf_fee: bool = True,
        apply_subscription: ApplySubscription = ApplySubscription.NONE,
        backtest_version: BacktestVersion = BacktestVersion.V2,
        slippage_percent: float = 0.0001,
        spread_markup: float = 0.0,
        start_date: str | None = None,
        end_date: str | None = None,
        broker: Broker = Broker.ALPACA_WHITE_LABEL,
        benchmark_symphonies: list[str] | None = None,
        benchmark_tickers: list[str] | None = None,
        sparkgraph_color: str | None = None,
    ) -> BacktestResult:
        """
        Run a backtest for a public symphony by its ID (v2).

        Args:
            symphony_id: The ID of the symphony to backtest.
            capital: Initial capital for the backtest (default: 10000.0).
            abbreviate_days: Number of days to abbreviate the backtest (for testing).
            apply_reg_fee: Whether to apply regulatory fees (SEC fees).
            apply_taf_fee: Whether to apply TAF (Trading Activity Fee).
            apply_subscription: Composer subscription level to simulate (affects fees).
            backtest_version: Backtest engine version to use.
            slippage_percent: Slippage assumption as decimal (0.0001 = 0.01%).
            spread_markup: Bid-ask spread markup as decimal (0.001 = 0.1%).
            start_date: Backtest start date (YYYY-MM-DD). Defaults to earliest available data.
            end_date: Backtest end date (YYYY-MM-DD). Defaults to latest available data.
            broker: Broker to simulate for fee calculations.
            benchmark_symphonies: List of symphony IDs to use as benchmarks.
            benchmark_tickers: List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ']).
            sparkgraph_color: Custom color for performance chart.

        Returns
        -------
            BacktestResult: Parsed backtest result with all statistics.

        Example:
             result = client.public_symphony.backtest_symphony_v2("sym-abc123")
             print(f"Sharpe: {result.stats.sharpe_ratio}")
        """
        payload = {
            "capital": capital,
            "abbreviate_days": abbreviate_days,
            "apply_reg_fee": apply_reg_fee,
            "apply_taf_fee": apply_taf_fee,
            "apply_subscription": apply_subscription.value if apply_subscription else None,
            "backtest_version": backtest_version.value if backtest_version else None,
            "slippage_percent": slippage_percent,
            "spread_markup": spread_markup,
            "start_date": start_date,
            "end_date": end_date,
            "broker": broker.value if broker else None,
            "benchmark_symphonies": benchmark_symphonies,
            "benchmark_tickers": benchmark_tickers,
            "sparkgraph_color": sparkgraph_color,
        }
        payload = {k: v for k, v in payload.items() if v is not None}

        raw_response = self._client.post(
            f"/api/v2/public/symphonies/{symphony_id}/backtest",
            json=payload,
        )
        return BacktestResult.model_validate(raw_response)



"""Conversation models - request and response models for conversation endpoints."""

from .requests import (
    CreateConversationRequest,
    FeedbackActionDetails,
    FeedbackRequest,
    FileItem,
    MessageContentItem,
    SendMessageRequest,
    UpdateStateRequest,
)
from .responses import (
    Conversation,
    ConversationMessage,
    ConversationResponse,
    ConversationState,
    DiscoverPrompt,
    StartingPoint,
    StartingPointsResponse,
)

__all__ = [
    "MessageContentItem",
    "FileItem",
    "CreateConversationRequest",
    "SendMessageRequest",
    "FeedbackActionDetails",
    "FeedbackRequest",
    "UpdateStateRequest",
    "ConversationResponse",
    "StartingPoint",
    "DiscoverPrompt",
    "StartingPointsResponse",
    "ConversationState",
    "ConversationMessage",
    "Conversation",
]



"""AI Agents resource for AI agent endpoints."""

from ..models.ai_agents import (
    AIAgent,
    AIAgentsResponse,
    AIExecutionsResponse,
)


class AIAgents:
    """Resource for AI agent endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def list(
        self,
        broker_account_id: str,
        limit: int | None = None,
        offset: int | None = None,
    ) -> AIAgentsResponse:
        """
        List AI agents for a broker account.

        Args:
            broker_account_id: The broker account ID
            limit: Maximum number of results
            offset: Pagination offset

        Returns
        -------
            List of AI agents
        """
        params: dict[str, str] = {"broker_account_id": broker_account_id}
        if limit is not None:
            params["limit"] = str(limit)
        if offset is not None:
            params["offset"] = str(offset)
        response = self._client.get("/api/v1/ai-agents", params=params)
        return AIAgentsResponse.model_validate(response)

    def get(self, agent_id: str) -> AIAgent:
        """
        Get an AI agent by ID.

        Args:
            agent_id: The AI agent ID

        Returns
        -------
            AI agent details
        """
        response = self._client.get(f"/api/v1/ai-agents/{agent_id}")
        return AIAgent.model_validate(response)

    def get_executions(
        self,
        agent_id: str,
        limit: int | None = None,
        offset: int | None = None,
    ) -> AIExecutionsResponse:
        """
        List executions for an AI agent.

        Args:
            agent_id: The AI agent ID
            limit: Maximum number of results
            offset: Pagination offset

        Returns
        -------
            List of agent executions
        """
        params = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset
        response = self._client.get(
            f"/api/v1/ai-agents/{agent_id}/executions",
            params=params if params else None,
        )
        return AIExecutionsResponse.model_validate(response)



"""AI Agents models - response models for AI agent endpoints."""

from .responses import (
    AIAgent,
    AIAgentsResponse,
    AIAgentStatus,
    AIExecution,
    AIExecutionsResponse,
    AIExecutionStatus,
    AIExecutionType,
)

__all__ = [
    "AIAgentStatus",
    "AIAgent",
    "AIAgentsResponse",
    "AIExecutionStatus",
    "AIExecutionType",
    "AIExecution",
    "AIExecutionsResponse",
]



"""Reports resource for account reports endpoints."""

from typing import Any

from ..models.reports import ReportType


class Reports:
    """Resource for account reports endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def get(
        self,
        account_id: str,
        since: str,
        until: str,
        report_type: ReportType | None = None,
    ) -> dict[str, Any]:
        """
        Get a report for the given account.

        Args:
            account_id: The account UUID
            since: Start date for the report (ISO format)
            until: End date for the report (ISO format)
            report_type: Type of report (trade-activity or non-trade-activity)

        Returns
        -------
            Report data for the account
        """
        params = {
            "since": since,
            "until": until,
        }
        if report_type:
            params["report-type"] = report_type.value

        response = self._client.get(
            f"/api/v1/reports/{account_id}",
            params=params,
        )
        return response



"""Composer API Client.

This module provides the main client for interacting with the Composer API.
"""

from .http_client import HTTPClient, RetryConfig
from .resources.accounts import Accounts
from .resources.ai_agents import AIAgents
from .resources.auth_management import AuthManagement
from .resources.backtest import Backtest
from .resources.cash import Cash
from .resources.conversation import Conversation
from .resources.deploy import DeployResource
from .resources.dry_run import DryRun
from .resources.market_data import MarketData
from .resources.portfolio import Portfolio
from .resources.public_symphony import PublicSymphony
from .resources.public_user import PublicUser
from .resources.quotes import Quotes
from .resources.reports import Reports
from .resources.search import Search
from .resources.trading import Trading
from .resources.user import User
from .resources.user_symphonies import UserSymphonies
from .resources.user_symphony import UserSymphony
from .resources.watchlist import Watchlist

_DEFAULT_RETRY_CONFIG = RetryConfig()


class ComposerClient:
    """Main client for interacting with the Composer API.

    This client provides access to various Composer API resources including
    backtest, market data, trading, portfolio management, and more.

    Args:
        api_key: The API Key ID
        api_secret: The API Secret Key
        timeout: Request timeout in seconds (default: 30.0)
        retry_config: Configuration for retry behavior (default: 3 retries, 10s for 429, 3s for 5xx)
    """

    def __init__(
        self,
        api_key: str,
        api_secret: str,
        timeout: float = 30.0,
        retry_config: RetryConfig | None = _DEFAULT_RETRY_CONFIG,
    ):
        """
        Initialize the Composer Client.

        Args:
            api_key (str): The API Key ID
            api_secret (str): The API Secret Key
            timeout (float): Request timeout in seconds (default: 30.0)
            retry_config (RetryConfig | None): Configuration for retry behavior.
                Pass None to disable retries (default: RetryConfig())
        """
        self.http_client = HTTPClient(
            api_key, api_secret, timeout=timeout, retry_config=retry_config
        )
        # Separate HTTP clients for backtest API (different base URL)
        # Public client (no auth headers) for public endpoints
        # Authenticated client for endpoints requiring auth
        self.backtest_auth_client = HTTPClient(
            api_key,
            api_secret,
            base_url="https://backtest-api.composer.trade/",
            timeout=timeout,
            retry_config=retry_config,
        )
        # Stagehand API client for portfolio/account endpoints
        self.stagehand_auth_client = HTTPClient(
            api_key,
            api_secret,
            base_url="https://stagehand-api.composer.trade/",
            timeout=timeout,
            retry_config=retry_config,
        )
        # Trading API client for deploy and trading endpoints
        self.trading_auth_client = HTTPClient(
            api_key,
            api_secret,
            base_url="https://trading-api.composer.trade/",
            timeout=timeout,
            retry_config=retry_config,
        )

        # Initialize resources
        self.backtest = Backtest(self.backtest_auth_client)
        # Market data uses stagehand API
        self.market_data = MarketData(self.stagehand_auth_client)
        # Accounts, Portfolio, Cash, User, AI Agents, Conversation, Reports, Auth use stagehand API
        self.accounts = Accounts(self.stagehand_auth_client)
        self.portfolio = Portfolio(self.stagehand_auth_client)
        self.cash = Cash(self.stagehand_auth_client)
        self.user = User(self.stagehand_auth_client)
        self.ai_agents = AIAgents(self.stagehand_auth_client)
        self.conversation = Conversation(self.stagehand_auth_client)
        self.reports = Reports(self.stagehand_auth_client)
        self.auth_management = AuthManagement(self.stagehand_auth_client)
        self.search = Search(self.backtest_auth_client)
        # Quotes uses stagehand API (public endpoint)
        self.quotes = Quotes(self.stagehand_auth_client)

        # Public user endpoints
        self.public_user = PublicUser(self.stagehand_auth_client)

        # New backtest API resources
        # Public endpoints (no auth required)
        self.public_symphony = PublicSymphony(self.backtest_auth_client)
        # Authenticated endpoints
        self.user_symphony = UserSymphony(self.backtest_auth_client)
        self.user_symphonies = UserSymphonies(self.backtest_auth_client)
        self.watchlist = Watchlist(self.backtest_auth_client)
        # Trading API resources (deploy, dry-run, and trading endpoints)
        self.deploy = DeployResource(self.trading_auth_client)
        self.dry_run = DryRun(self.trading_auth_client)
        self.trading = Trading(self.trading_auth_client)



"""Deploy models - request and response models for deploy endpoints."""

from .responses import (
    Deploy,
    DeployActionResponse,
    DeploysResponse,
    DeploySymphoniesResponse,
    DeploySymphony,
    MarketHoursItem,
    MarketHoursResponse,
)

__all__ = [
    "MarketHoursItem",
    "MarketHoursResponse",
    "DeploySymphony",
    "DeploySymphoniesResponse",
    "Deploy",
    "DeploysResponse",
    "DeployActionResponse",
]



"""Market data resource for options and other market data."""

from ..models.market_data import (
    ContractType,
    CustomBars,
    MarketOverview,
    MarketSnapshot,
    OptionsChainResponse,
    OptionsContractResponse,
    OptionSortBy,
    OptionsOverview,
    SortOrder,
    TopMoversResponse,
)


class MarketData:
    """Resource for market data endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def get_options_chain(
        self,
        underlying: str,
        next_cursor: str | None = None,
        strike_price: float | None = None,
        expiry: str | None = None,
        contract_type: ContractType | None = None,
        order: SortOrder = SortOrder.ASC,
        limit: int | None = None,
        sort_by: OptionSortBy = OptionSortBy.SYMBOL,
    ) -> OptionsChainResponse:
        """
        Get options chain for a specific underlying asset.

        Args:
            underlying: The underlying asset symbol (e.g., "AAPL")
            next_cursor: Pagination cursor for fetching the next page
            strike_price: Filter by specific strike price
            expiry: Filter by expiration date (YYYY-MM-DD format)
            contract_type: Filter by option type (CALL or PUT)
            order: Sort order for results
            limit: Maximum number of results to return (max 250)
            sort_by: Field to sort results by

        Returns
        -------
            OptionsChainResponse with list of options contracts
        """
        params: dict = {
            "underlying_asset_symbol": underlying,
            "order": order.value,
            "sort_by": sort_by.value,
        }
        if next_cursor:
            params["next_cursor"] = next_cursor
        if strike_price:
            params["strike_price"] = strike_price
        if expiry:
            params["expiry"] = expiry
        if contract_type:
            params["contract_type"] = contract_type.value
        if limit:
            params["limit"] = limit

        response = self._client.get("/api/v1/market-data/options/chain", params=params)
        return OptionsChainResponse.model_validate(response)

    def get_options_contract(self, symbol: str) -> OptionsContractResponse:
        """
        Get market data for a specific options contract.

        Args:
            symbol: The options contract symbol (e.g., "OPTIONS::AAPL250718C00210000//USD")

        Returns
        -------
            OptionsContractResponse with contract market data
        """
        response = self._client.get(
            "/api/v1/market-data/options/contract", params={"symbol": symbol}
        )

        return OptionsContractResponse.model_validate(response)

    def get_options_overview(self, symbol: str) -> OptionsOverview:
        """
        Get an overview of options data for a given ticker.

        Args:
            symbol: The underlying asset symbol (e.g., "AAPL")

        Returns
        -------
            OptionsOverview with available expiration dates
        """
        response = self._client.get(
            "/api/v1/market-data/options/overview", params={"symbol": symbol}
        )
        return OptionsOverview.model_validate(response)

    def get_snapshot(self, symbol: str) -> MarketSnapshot:
        """
        Get a snapshot of live market data.

        Args:
            symbol: The symbol to get snapshot for (e.g., "AAPL", "BTC-USD")

        Returns
        -------
            MarketSnapshot with bid, ask, last trade, and change data
        """
        response = self._client.get("/api/v1/market-data/snapshot", params={"symbol": symbol})
        return MarketSnapshot.model_validate(response)

    def get_custom_bars(
        self,
        symbol: str,
        range_date_from: str | None = None,
        range_date_to: str | None = None,
        range_preset: str | None = None,
    ) -> CustomBars:
        """
        Get custom bars market data.

        Args:
            symbol: The symbol to get bars for (e.g., "AAPL", "BTC-USD")
            range_date_from: Start date (YYYY-MM-DD format)
            range_date_to: End date (YYYY-MM-DD format)
            range_preset: Preset range (e.g., "1M", "3M", "1Y", "5Y")

        Returns
        -------
            CustomBars with OHLCV data
        """
        params = {"symbol": symbol}
        if range_date_from:
            params["range_date_from"] = range_date_from
        if range_date_to:
            params["range_date_to"] = range_date_to
        if range_preset:
            params["range_preset"] = range_preset

        response = self._client.get("/api/v1/market-data/custom-bars", params=params)
        return CustomBars.model_validate(response)

    def get_market_overview(self, symbol: str) -> MarketOverview:
        """
        Get an overview of reference data for a symbol.

        Args:
            symbol: The symbol to get overview for (e.g., "AAPL", "BTC-USD")

        Returns
        -------
            MarketOverview with company info, market cap, etc.
        """
        response = self._client.get("/api/v1/market-data/overview", params={"symbol": symbol})
        return MarketOverview.model_validate(response)

    def get_top_movers(self) -> TopMoversResponse:
        """
        Get top movers market data.

        Returns
        -------
            TopMoversResponse with list of top gaining/losing symbols
        """
        response = self._client.get("/api/v1/market-data/top-movers")
        return TopMoversResponse.model_validate(response)



"""Cash resource for cash-related endpoints."""

from typing import Any

from ..models.cash import (
    ACHLimits,
    ACHRelationshipsResponse,
    ACHTransfer,
    RecurringDepositProjection,
    RecurringDepositsMeta,
    RecurringDepositsResponse,
    TaxWithholding,
    TransferConstraints,
)


class Cash:
    """Resource for cash-related endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def get_transfer_constraints(self) -> dict[str, TransferConstraints]:
        """
        Get all transfer constraints for a user's active accounts.

        Returns
        -------
            Map of account UUID to transfer constraints
        """
        response = self._client.get("/api/v1/cash/transfer-constraints")
        return {
            account_id: TransferConstraints.model_validate(constraints)
            for account_id, constraints in response.items()
        }

    def get_ach_relationships(
        self,
        include_plaid_account_details: bool,
    ) -> ACHRelationshipsResponse:
        """
        Get all ACH relationships for a user.

        Args:
            include_plaid_account_details: Whether to include Plaid account details

        Returns
        -------
            ACH relationships including bank accounts
        """
        params = {"include_plaid_account_details": include_plaid_account_details}
        response = self._client.get(
            "/api/v1/cash/ach-relationships",
            params=params,
        )
        return ACHRelationshipsResponse.model_validate(response)

    def get_ach_limits(self, account_id: str) -> ACHLimits:
        """
        Get limits to ACH transfers.

        Args:
            account_id: The account UUID

        Returns
        -------
            ACH transfer limits
        """
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/ach-limits",
        )
        return ACHLimits.model_validate(response)

    def get_tax_withholding_federal(self, account_id: str) -> TaxWithholding:
        """
        Get federal tax withholding requirements for Traditional IRA distributions.

        Args:
            account_id: The account UUID

        Returns
        -------
            Tax withholding information
        """
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/tax-withholding/federal",
        )
        return TaxWithholding.model_validate(response)

    def get_ach_transfers(
        self,
        account_id: str,
        year: int | None = None,
    ) -> list[ACHTransfer]:
        """
        Get an account's ACH transfers.

        Args:
            account_id: The account UUID
            year: Optional year to filter transfers

        Returns
        -------
            List of ACH transfers
        """
        params = {}
        if year is not None:
            params["year"] = year
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/ach-transfers",
            params=params if params else None,
        )
        return [ACHTransfer.model_validate(t) for t in response]

    def get_recurring_deposits(
        self,
        account_id: str,
        status: str | None = None,
        n: int = 10,
    ) -> RecurringDepositsResponse:
        """
        Get all recurring deposits for a broker account.

        Args:
            account_id: The account UUID
            status: Optional status filter (ACTIVE, CANCELED, etc.)
            n: Number of results to return

        Returns
        -------
            List of recurring deposits
        """
        params: dict[str, Any] = {"n": n}
        if status:
            params["status"] = status
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/recurring-deposits",
            params=params,
        )
        return RecurringDepositsResponse.model_validate(response)

    def get_recurring_deposits_meta(self, account_id: str) -> RecurringDepositsMeta:
        """
        Get maximum recurring deposit amount for all frequencies.

        Args:
            account_id: The account UUID

        Returns
        -------
            Max deposit amounts by frequency
        """
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/recurring-deposits-meta",
        )
        return RecurringDepositsMeta.model_validate(response)

    def get_recurring_deposits_projection(
        self,
        account_id: str,
        amount: float,
        frequency: str,
    ) -> RecurringDepositProjection:
        """
        Project when a retirement account would reach its annual contribution limit.

        Args:
            account_id: The account UUID
            amount: Deposit amount
            frequency: Deposit frequency (WEEKLY, SEMIMONTHLY, MONTHLY, QUARTERLY)

        Returns
        -------
            Projection of when limit will be hit
        """
        params = {"amount": amount, "frequency": frequency}
        response = self._client.get(
            f"/api/v1/cash/accounts/{account_id}/recurring-deposits-projection",
            params=params,
        )
        return RecurringDepositProjection.model_validate(response)

    def get_all_recurring_deposits(
        self,
        n: int = 10,
        status: str | None = None,
    ) -> RecurringDepositsResponse:
        """
        Get all recurring deposits for a user.

        Args:
            n: Number of results to return
            status: Optional status filter (ACTIVE, CANCELED, etc.)

        Returns
        -------
            List of all recurring deposits across accounts
        """
        params: dict[str, Any] = {"n": n}
        if status:
            params["status"] = status
        response = self._client.get(
            "/api/v1/cash/recurring-deposits",
            params=params,
        )
        return RecurringDepositsResponse.model_validate(response)



"""Composer API models organized by endpoint sections."""

# Common models (shared across all sections)
# Accounts section
from .accounts import (
    Account,
    AccountInfo,
    AccountsListResponse,
    AccountType,
    Address,
    AssetClass,
    AvailableAccountType,
    BuyingPower,
    Contact,
    DirectTradableAssetClasses,
    FootprintPlaybook,
    Holding,
    Identity,
    IncomeRange,
    InvestorDocument,
    InvestorDocumentCategory,
    InvestorProfile,
    OptionsDetails,
    SupportedRegionsResponse,
    SupportedRegionStates,
    Trade,
    TradeHistoryItem,
    TradeVolumeResponse,
    TrustedContact,
)

# Backtest section
from .backtest import (
    AISymphonyDescription,
    ApplySubscription,
    AssetClass as BacktestAssetClass,
    BacktestExistingSymphonyRequest,
    # Request models
    BacktestRequest,
    BacktestResult,
    # Enums
    BacktestVersion,
    Broker,
    # Response models
    Costs,
    DataWarning,
    DraftSymphoniesResponse,
    DraftSymphony,
    Indicator,
    IndicatorParameter,
    LegendEntry,
    ParameterType,
    Quote,
    RebalanceRequest,
    RebalanceResult,
    # Symphony models
    SymphonyDetail,
    SymphonyMeta,
    SymphonyMetaResponse,
    SymphonyRebalanceState,
    SymphonyRunResult,
    SymphonyTickersResponse,
    SymphonyVersionInfo,
    TickersResponse,
    UserSymphoniesResponse,
    UserSymphony,
    WatchlistResponse,
    WatchlistSymphony,
)

# Cash section
from .cash import (
    ACHLimits,
    ACHRelationship,
    ACHRelationshipsResponse,
    ACHTransfer,
    Frequency,
    RecurringDeposit,
    RecurringDepositMeta,
    RecurringDepositProjection,
    RecurringDepositProjectionReason,
    RecurringDepositsMeta,
    RecurringDepositsResponse,
    RecurringDepositStatus,
    TaxWithholding,
    TransferConstraints,
)
from .common import (
    Asset,
    BaseNode,
    BenchmarkStats,
    Empty,
    Filter,
    # Symphony types
    Function,
    Group,
    If,
    IfChildFalse,
    IfChildTrue,
    RebalanceFrequency,
    RegressionMetrics,
    Stats,
    SymphonyDefinition,
    SymphonyScore,
    WeightCashEqual,
    WeightCashSpecified,
    WeightInverseVol,
    WeightMap,
    validate_symphony_score,
)

# Dry Run section
from .dry_run import (
    AccountDryRunResult,
    DryRunRequest,
    DryRunResult,
    PreviewRecommendedTrade,
    RecommendedTrade,
    TradePreviewRequest,
    TradePreviewResult,
    TradeSide,
)

# Market Data section
from .market_data import (
    ContractType,
    OptionContract,
    OptionContractDetails,
    OptionDayData,
    OptionGreeks,
    OptionsChainResponse,
    OptionsContractResponse,
    OptionSortBy,
    OptionsOverview,
    PayoffStatus,
    QuotePrice,
    SortOrder,
)

# Portfolio section
from .portfolio import (
    ActivityHistoryItem,
    ActivityHistoryResponse,
    DeployHistoryItem,
    DeployHistoryResponse,
    HoldingAllocation,
    HoldingStats,
    HoldingStatsResponse,
    PortfolioHistory,
    PositionDirection,
    SymphonyAllocation,
    SymphonyHolding,
    SymphonyHoldingAllocation,
    SymphonyHoldings,
    SymphonyStats,
    SymphonyStatsMeta,
    SymphonyStatsMetaResponse,
    SymphonyStatsResponse,
    TimeSeries,
    TotalStats,
)

# Reports section
from .reports import ReportType

# Search section
from .search import (
    SearchSymphoniesRequest,
    SearchSymphonyResult,
    SymphonyAIDescription,
)

# Symphony section
from .symphony import (
    AssetClass as SymphonyAssetClass,
    Benchmark,
    BenchmarkType,
    CopySymphonyRequest,
    CopySymphonyResponse,
    CreateSymphonyRequest,
    CreateSymphonyResponse,
    SymphonyVersion,
    UpdateSymphonyNodesResponse,
    UpdateSymphonyResponse,
)

# Trading section
from .trading import (
    AssetClass as TradingAssetClass,
    CreateOrderRequest,
    CreateOrderResponse,
    ListOrdersResponse,
    OrderRequest,
    OrderSide,
    OrderSource,
    OrderStatus,
    OrderType,
    PositionIntent,
    TimeInForce,
)

# User section
from .user import (
    AgreementStatusResponse,
    JWTResponse,
    UserOnboardingExtraInfo,
    UserOnboardingState,
    UserOnboardingStateAccount,
    UserPreferences,
    UserPreferencesBanners,
    UserPreferencesOnboarding,
    UserPreferencesPortfolioGraph,
    UserPreferencesSymphonyTable,
    UserProfile,
)

# For backward compatibility, also export old model names
# These are deprecated and will be removed in a future version
BacktestResponse = BacktestResult

__all__ = [
    # Common stats
    "Stats",
    "BenchmarkStats",
    "RegressionMetrics",
    # Symphony types
    "Function",
    "RebalanceFrequency",
    "WeightMap",
    "BaseNode",
    "Asset",
    "Empty",
    "If",
    "IfChildTrue",
    "IfChildFalse",
    "Filter",
    "WeightInverseVol",
    "Group",
    "WeightCashEqual",
    "WeightCashSpecified",
    "SymphonyScore",
    "SymphonyDefinition",
    "validate_symphony_score",
    # Accounts
    "AssetClass",
    "DirectTradableAssetClasses",
    "Account",
    "Holding",
    "InvestorDocument",
    "InvestorDocumentCategory",
    "AccountsListResponse",
    "AccountType",
    "AvailableAccountType",
    "SupportedRegionsResponse",
    "SupportedRegionStates",
    "FootprintPlaybook",
    "Trade",
    "TradeVolumeResponse",
    "TradeHistoryItem",
    "Identity",
    "Address",
    "Contact",
    "IncomeRange",
    "InvestorProfile",
    "TrustedContact",
    "AccountInfo",
    "BuyingPower",
    # Portfolio
    "PositionDirection",
    "HoldingAllocation",
    "SymphonyAllocation",
    "SymphonyHoldingAllocation",
    "HoldingStats",
    "HoldingStatsResponse",
    "TotalStats",
    "SymphonyHolding",
    "SymphonyStats",
    "SymphonyStatsMeta",
    "SymphonyStatsMetaResponse",
    "SymphonyStatsResponse",
    "TimeSeries",
    "PortfolioHistory",
    "SymphonyHoldings",
    "ActivityHistoryItem",
    "ActivityHistoryResponse",
    "DeployHistoryItem",
    "DeployHistoryResponse",
    # Cash
    "TransferConstraints",
    "ACHRelationship",
    "ACHRelationshipsResponse",
    "ACHLimits",
    "TaxWithholding",
    "ACHTransfer",
    "Frequency",
    "RecurringDepositStatus",
    "RecurringDeposit",
    "RecurringDepositsResponse",
    "RecurringDepositMeta",
    "RecurringDepositsMeta",
    "RecurringDepositProjectionReason",
    "RecurringDepositProjection",
    # User
    "JWTResponse",
    "UserPreferences",
    "UserPreferencesOnboarding",
    "UserPreferencesPortfolioGraph",
    "UserPreferencesSymphonyTable",
    "UserPreferencesBanners",
    "UserOnboardingStateAccount",
    "UserOnboardingState",
    "UserOnboardingExtraInfo",
    "UserProfile",
    "AgreementStatusResponse",
    # Reports
    "ReportType",
    # Search
    "SymphonyAIDescription",
    "SearchSymphonyResult",
    "SearchSymphoniesRequest",
    # Symphony
    "SymphonyAssetClass",
    "BenchmarkType",
    "Benchmark",
    "CreateSymphonyRequest",
    "CreateSymphonyResponse",
    "CopySymphonyRequest",
    "CopySymphonyResponse",
    "UpdateSymphonyResponse",
    "UpdateSymphonyNodesResponse",
    "SymphonyVersion",
    # Trading
    "OrderStatus",
    "OrderSide",
    "OrderType",
    "TimeInForce",
    "PositionIntent",
    "TradingAssetClass",
    "OrderSource",
    "OptionsDetails",
    "OrderRequest",
    "CreateOrderRequest",
    "CreateOrderResponse",
    "ListOrdersResponse",
    # Market Data
    "ContractType",
    "PayoffStatus",
    "OptionSortBy",
    "SortOrder",
    "OptionGreeks",
    "QuotePrice",
    "OptionDayData",
    "OptionContractDetails",
    "OptionContract",
    "OptionsChainResponse",
    "OptionsContractResponse",
    "OptionsOverview",
    # Dry Run
    "TradeSide",
    "RecommendedTrade",
    "PreviewRecommendedTrade",
    "DryRunResult",
    "TradePreviewResult",
    "AccountDryRunResult",
    "DryRunRequest",
    "TradePreviewRequest",
    # Backtest enums
    "BacktestVersion",
    "Broker",
    "ApplySubscription",
    "ParameterType",
    "BacktestAssetClass",
    # Backtest symphony models
    "IndicatorParameter",
    "Indicator",
    "SymphonyMeta",
    "SymphonyMetaResponse",
    "SymphonyDetail",
    "SymphonyVersionInfo",
    "TickersResponse",
    "AISymphonyDescription",
    "UserSymphony",
    "DraftSymphony",
    "WatchlistSymphony",
    "UserSymphoniesResponse",
    "DraftSymphoniesResponse",
    "WatchlistResponse",
    "SymphonyTickersResponse",
    # Backtest models
    "BacktestRequest",
    "BacktestExistingSymphonyRequest",
    "Quote",
    "SymphonyRebalanceState",
    "RebalanceRequest",
    "Costs",
    "DataWarning",
    "LegendEntry",
    "BacktestResult",
    "SymphonyRunResult",
    "RebalanceResult",
    # Backward compatibility
    "BacktestResponse",
]



"""Common models shared across all API sections."""

from .stats import BenchmarkStats, RegressionMetrics, Stats
from .symphony import (
    Asset,
    BaseNode,
    Empty,
    Filter,
    Function,
    Group,
    If,
    IfChildFalse,
    IfChildTrue,
    RebalanceFrequency,
    SymphonyDefinition,
    SymphonyScore,
    WeightCashEqual,
    WeightCashSpecified,
    WeightInverseVol,
    WeightMap,
    validate_symphony_score,
)

__all__ = [
    # Stats
    "Stats",
    "BenchmarkStats",
    "RegressionMetrics",
    # Symphony
    "Function",
    "RebalanceFrequency",
    "WeightMap",
    "BaseNode",
    "Asset",
    "Empty",
    "If",
    "IfChildTrue",
    "IfChildFalse",
    "Filter",
    "WeightInverseVol",
    "Group",
    "WeightCashEqual",
    "WeightCashSpecified",
    "SymphonyScore",
    "SymphonyDefinition",
    "validate_symphony_score",
]



"""Accounts response models."""

from enum import StrEnum

from pydantic import BaseModel, Field


class AssetClass(StrEnum):
    """Asset class types."""

    CRYPTO = "CRYPTO"
    EQUITIES = "EQUITIES"
    OPTIONS = "OPTIONS"


class DirectTradableAssetClasses(BaseModel):
    """Asset classes available for direct trading."""

    model_config = {"populate_by_name": True}

    buy: list[AssetClass] = Field(
        description="Asset classes available for buying via direct trading"
    )
    sell: list[AssetClass] = Field(
        description="Asset classes available for selling via direct trading"
    )


class Account(BaseModel):
    """
    A brokerage account.

    Contains information about the account status, supported asset classes,
    and key dates for account activity.
    """

    model_config = {"populate_by_name": True}

    account_uuid: str = Field(description="Unique identifier (UUID) for the account")
    account_foreign_id: str | None = Field(
        None, description="Foreign ID from the broker (if applicable)"
    )
    account_type: str = Field(description="Type of account (e.g., INDIVIDUAL, IRA, etc.)")
    asset_classes: list[AssetClass] = Field(description="Asset classes supported by this account")
    direct_tradable_asset_classes: DirectTradableAssetClasses = Field(
        description="Asset classes available for direct buy/sell trading"
    )
    symphony_tradable_asset_classes: list[AssetClass] = Field(
        description="Asset classes available for Symphony automation"
    )
    account_number: str | None = Field(None, description="Account number (if available)")
    status: str = Field(description="Account status (e.g., ACTIVE, PENDING, etc.)")
    broker: str = Field(description="Broker handling this account (e.g., ALPACA, APEX)")
    created_at: str = Field(description="When the account was created (ISO 8601 format)")
    first_deposit_at: str | None = Field(None, description="When the first deposit was made")
    first_incoming_acats_transfer_at: str | None = Field(
        None, description="When the first incoming ACATS transfer occurred"
    )
    first_deploy_at: str | None = Field(None, description="When the first symphony was deployed")
    first_position_created_at: str | None = Field(
        None, description="When the first position was created"
    )
    is_legacy_crypto_only_account: bool | None = Field(
        None, description="Whether this is a legacy crypto-only account"
    )
    has_queued_deploy: bool = Field(description="Whether there are any queued symphony deployments")
    has_active_position: bool = Field(description="Whether the account has any active positions")


class OptionsDetails(BaseModel):
    """Details for options contract holdings."""

    model_config = {"populate_by_name": True}

    underlying_asset_symbol: str = Field(description="Underlying asset symbol (e.g., AAPL)")
    strike_price: float = Field(description="Strike price of the option contract")
    expiry: str = Field(description="Expiry date (YYYY-MM-DD format)")
    contract_type: str = Field(description="Type of option: CALL or PUT")


class Holding(BaseModel):
    """
    A position/holding in an account.

    Represents equities, crypto, or options positions.
    """

    model_config = {"populate_by_name": True}

    ticker: str = Field(description="Ticker symbol (e.g., AAPL, CRYPTO::BTC//USD)")
    quantity: float = Field(description="Number of shares/contracts/units held")
    asset_class: AssetClass = Field(description="Type of asset: CRYPTO, EQUITIES, or OPTIONS")
    options_details: OptionsDetails | None = Field(
        None, description="Details for options contracts (only for OPTIONS)"
    )


class InvestorDocumentCategory(StrEnum):
    """Categories of investor documents."""

    STATEMENT = "STATEMENT"
    CRYPTO_ACCOUNT_STATEMENT = "CRYPTO_ACCOUNT_STATEMENT"
    CRYPTO_TAX_FORM = "CRYPTO_TAX_FORM"
    TAX_FORM = "TAX_FORM"
    TRADE_CONFIRMATION = "TRADE_CONFIRMATION"


class InvestorDocument(BaseModel):
    """
    An investor document from the broker.

    Includes tax documents, account statements, trade confirmations, etc.
    """

    model_config = {"populate_by_name": True}

    category: InvestorDocumentCategory = Field(description="Category of the document")
    date: str = Field(description="Date of the document")
    id: str = Field(description="Unique identifier for the document")
    type: str = Field(description="Type of document")
    url: str = Field(description="URL to download the document")
    file_name: str = Field(description="File name of the document")
    tax_ids: list[str] | None = Field(None, description="Tax IDs associated with the document")


class AccountsListResponse(BaseModel):
    """Response from listing all accounts."""

    model_config = {"populate_by_name": True}

    accounts: list[Account] = Field(description="List of all accounts for the authenticated user")


class AccountType(StrEnum):
    """Account types available for creation."""

    INDIVIDUAL = "INDIVIDUAL"
    ROTH_IRA = "ROTH_IRA"
    TRADITIONAL_IRA = "TRADITIONAL_IRA"
    ROLLOVER_IRA = "ROLLOVER_IRA"


class FootprintPlaybook(BaseModel):
    """Footprint playbook for account creation."""

    model_config = {"populate_by_name": True}

    purpose: str
    playbook_key: str
    index: int


class AvailableAccountType(BaseModel):
    """Available account type for a specific region."""

    model_config = {"populate_by_name": True}

    account_type: AccountType
    asset_classes: list[AssetClass]
    options_trading_levels: list[int] | None = None
    footprint_kyc_completed: bool
    footprint_token: str | None = None
    footprint_playbook_key: str | None = None
    footprint_playbooks: list[FootprintPlaybook] = []


class SupportedRegionStates(BaseModel):
    """States supporting trading for a specific asset class."""

    model_config = {"populate_by_name": True}

    countries: list[str] = []
    states: dict[str, list[str]] = {}


class SupportedRegionsResponse(BaseModel):
    """Response from getting supported regions."""

    model_config = {"populate_by_name": True}

    EQUITIES: SupportedRegionStates
    CRYPTO: SupportedRegionStates


class Trade(BaseModel):
    """A historical trade record."""

    model_config = {"populate_by_name": True}

    order_id: str
    order_request_id: str | None = None
    status: str
    side: str | None = None
    symbol: str | None = None
    asset_class: AssetClass | None = None
    filled_avg_price: float | None = None
    filled_qty: float | None = None
    filled_notional: float | None = None
    created_at: str


class TradeVolumeResponse(BaseModel):
    """Response from getting trade volume."""

    model_config = {"populate_by_name": True}

    volume: float = Field(description="Total trade volume in the account")


class TradeHistoryItem(BaseModel):
    """A trade history item including option events."""

    model_config = {"populate_by_name": True}

    id: str
    status: str
    symbol: str | None = None
    asset_class: AssetClass | None = None
    unit_price: float | None = None
    qty: float | None = None
    net_amount: float | None = None
    created_at: str
    description: str | None = None
    activity_type: str
    side: str | None = None
    order_request_id: str | None = None


class Identity(BaseModel):
    """Account owner identity information."""

    model_config = {"populate_by_name": True}

    given_name: str
    middle_name: str | None = None
    family_name: str


class Address(BaseModel):
    """Postal address."""

    model_config = {"populate_by_name": True}

    street_address: list[str]
    city: str
    state: str | None = None
    postal_code: str
    country: str | None = None


class Contact(BaseModel):
    """Contact information."""

    model_config = {"populate_by_name": True}

    email_address: str
    phone_number: str | None = None
    address: Address


class IncomeRange(BaseModel):
    """Income range in USD."""

    model_config = {"populate_by_name": True}

    min: int
    max: int


class InvestorProfile(BaseModel):
    """Investor profile information."""

    model_config = {"populate_by_name": True}

    investment_objective: str | None = None
    annual_income_usd: IncomeRange
    total_net_worth_usd: IncomeRange


class TrustedContact(BaseModel):
    """Trusted contact information."""

    model_config = {"populate_by_name": True}

    given_name: str
    family_name: str
    email_address: str | None = None
    phone_number: str | None = None
    mailing_address: Address | None = None


class AccountInfo(BaseModel):
    """Basic account owner information."""

    model_config = {"populate_by_name": True}

    account_number: str | None = None
    account_type: str
    broker: str
    status: str | None = None
    broker_status: str | None = None
    identity: Identity
    contact: Contact
    investor_profile: InvestorProfile | None = None
    trusted_contact: TrustedContact | None = None
    pending_trusted_contact: bool | None = None


class BuyingPower(BaseModel):
    """Buying power information for an asset class."""

    model_config = {"populate_by_name": True}

    asset_class: AssetClass
    total_cash: float
    total_unallocated_cash: float
    cash_reserved_for_options_contracts: float
    pending_deploys_cash: float
    pending_withdrawals: float | None = None
    pending_net_deposits: float | None = None
    buying_power: float
    symphony_buying_power: float
    direct_trading_buying_power: float
    direct_trading_daytrading_buying_power: float
    pattern_day_trader_eligible: bool
    pattern_day_trader: bool
    daytrading_blocked: bool
    daytrade_margin_call_protection_on_entry: bool
    daytrade_margin_call_protection_on_exit: bool
    daytrading_buying_power: float | None = None
    current_equity_for_dtbp: float | None = None
    sod_equity_for_dtbp: float | None = None
    buying_power_constrained_when_daytrading: bool



"""Backtest API models - indicators, symphonies, versions, watchlist."""

from enum import StrEnum

from pydantic import BaseModel, Field


class ParameterType(StrEnum):
    """Type of indicator parameter."""

    DOUBLE = "double"
    INT = "int"


class IndicatorParameter(BaseModel):
    """Parameter definition for a technical indicator."""

    key: str
    name: str
    type: ParameterType
    default_value: float = Field(alias="default_value")
    min_value: float = Field(alias="min_value")
    max_value: float = Field(alias="max_value")


class Indicator(BaseModel):
    """
    Technical indicator available for symphony conditions.

    Examples
    --------
        - Moving average of price
        - Relative strength index
        - Standard deviation
    """

    key: str
    composer_code_name: str = Field(alias="composer_code_name")
    name: str
    builder_format_string: str = Field(alias="builder_format_string")
    url: str | None = None
    description: str | None = None
    beta: bool = False
    parameters: list[IndicatorParameter]
    unit: str  # "dollars" or "percent"
    asset_only: bool = Field(alias="asset_only")


class AssetClass(StrEnum):
    """Asset class enum."""

    EQUITIES = "EQUITIES"
    CRYPTO = "CRYPTO"


class SymphonyMeta(BaseModel):
    """Basic symphony metadata."""

    id: str
    asset_class: AssetClass = Field(default=AssetClass.EQUITIES)
    asset_classes: list[AssetClass]
    name: str | None = None
    color: str
    description: str | None = None
    last_semantic_update_at: str = Field(alias="last_semantic_update_at")
    tags: list[str]
    rebalance_frequency: str = Field(alias="rebalance_frequency")
    is_shared: bool = Field(alias="is_shared")
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)


class SymphonyMetaResponse(BaseModel):
    """Response for getting symphony metadata."""

    symphonies: list[SymphonyMeta]


class SymphonyDetail(BaseModel):
    """Detailed symphony information."""

    symphony_id: int | None = Field(alias="symphony_id", default=None)
    id: str | None = Field(alias="id", default=None)
    created_at: str | None = Field(alias="created_at", default=None)
    updated_at: str | None = Field(alias="updated_at", default=None)
    asset_class: AssetClass | None = None
    asset_classes: list[AssetClass] | None = Field(default_factory=list)
    user_id: int | None = Field(alias="user_id", default=None)
    user_sid: str | None = Field(alias="user_sid", default=None)
    community_review_status: str | None = Field(alias="community_review_status", default=None)
    copied_from_symphony_id: int | None = Field(alias="copied_from_symphony_id", default=None)
    copied_from_symphony_sid: str | None = Field(alias="copied_from_symphony_sid", default=None)
    is_discover_symphony: bool | None = Field(alias="is_discover_symphony", default=None)
    is_deleted: bool | None = Field(alias="is_deleted", default=None)
    last_semantic_update_at: str | None = Field(alias="last_semantic_update_at", default=None)
    name: str | None = None
    description: str | None = None
    rebalance_frequency: str | None = Field(alias="rebalance_frequency", default=None)
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)
    is_shared: bool | None = Field(alias="is_shared", default=None)
    is_searchable: bool | None = Field(alias="is_searchable", default=None)
    is_searchable_updated_at: str | None = Field(alias="is_searchable_updated_at", default=None)
    investor_count: int | None = Field(alias="investor_count", default=None)
    watcher_count: int | None = Field(alias="watcher_count", default=None)
    color: str | None = None
    hashtag: str | None = None
    tags: list[str] | None = Field(default_factory=list)
    sparkgraph_gcs_url: str | None = Field(alias="sparkgraph_gcs_url", default=None)
    last_backtest_first_market_day: str | None = Field(
        alias="last_backtest_first_market_day", default=None
    )
    last_backtest_last_market_day: str | None = Field(
        alias="last_backtest_last_market_day", default=None
    )
    last_backtest_holdings: dict[str, float] | None = Field(
        alias="last_backtest_holdings", default=None
    )
    last_backtest_value: float | None = Field(alias="last_backtest_value", default=None)
    stats_trailing_one_month_return: float | None = Field(
        alias="stats_trailing_one_month_return", default=None
    )
    stats_trailing_three_month_return: float | None = Field(
        alias="stats_trailing_three_month_return", default=None
    )
    stats_trailing_one_year_return: float | None = Field(
        alias="stats_trailing_one_year_return", default=None
    )
    stats_annualized_rate_of_return: float | None = Field(
        alias="stats_annualized_rate_of_return", default=None
    )
    stats_cumulative_return: float | None = Field(alias="stats_cumulative_return", default=None)
    stats_max_drawdown: float | None = Field(alias="stats_max_drawdown", default=None)
    stats_standard_deviation: float | None = Field(alias="stats_standard_deviation", default=None)
    stats_min: float | None = Field(alias="stats_min", default=None)
    stats_max: float | None = Field(alias="stats_max", default=None)
    stats_mean: float | None = Field(alias="stats_mean", default=None)
    stats_size: int | None = Field(alias="stats_size", default=None)
    stats_median: float | None = Field(alias="stats_median", default=None)
    stats_sharpe_ratio: float | None = Field(alias="stats_sharpe_ratio", default=None)
    stats_calmar_ratio: float | None = Field(alias="stats_calmar_ratio", default=None)
    stats_oos_spy_beta: float | None = Field(alias="stats_oos_spy_beta", default=None)
    stats_oos_spy_alpha: float | None = Field(alias="stats_oos_spy_alpha", default=None)
    stats_oos_spy_pearson_r: float | None = Field(alias="stats_oos_spy_pearson_r", default=None)
    stats_oos_btcusd_beta: float | None = Field(alias="stats_oos_btcusd_beta", default=None)
    stats_oos_btcusd_alpha: float | None = Field(alias="stats_oos_btcusd_alpha", default=None)
    stats_oos_btcusd_pearson_r: float | None = Field(
        alias="stats_oos_btcusd_pearson_r", default=None
    )
    stats_oos_skewness: float | None = Field(alias="stats_oos_skewness", default=None)
    stats_oos_kurtosis: float | None = Field(alias="stats_oos_kurtosis", default=None)
    stats_oos_sortino_ratio: float | None = Field(alias="stats_oos_sortino_ratio", default=None)
    stats_oos_annualized_turnover: float | None = Field(
        alias="stats_oos_annualized_turnover", default=None
    )
    stats_oos_trailing_one_day_return: float | None = Field(
        alias="stats_oos_trailing_one_day_return", default=None
    )
    stats_oos_trailing_one_week_return: float | None = Field(
        alias="stats_oos_trailing_one_week_return", default=None
    )
    stats_oos_trailing_two_week_return: float | None = Field(
        alias="stats_oos_trailing_two_week_return", default=None
    )
    stats_oos_top_one_day_contribution: float | None = Field(
        alias="stats_oos_top_one_day_contribution", default=None
    )
    stats_oos_top_five_percent_day_contribution: float | None = Field(
        alias="stats_oos_top_five_percent_day_contribution", default=None
    )
    stats_oos_top_ten_percent_day_contribution: float | None = Field(
        alias="stats_oos_top_ten_percent_day_contribution", default=None
    )
    stats_oos_herfindahl_index: float | None = Field(
        alias="stats_oos_herfindahl_index", default=None
    )
    stats_oos_win_rate: float | None = Field(alias="stats_oos_win_rate", default=None)
    stats_oos_tail_ratio: float | None = Field(alias="stats_oos_tail_ratio", default=None)
    youtube_url: str | None = None
    categories: list[str] | None = None
    risk_rating: str | None = None
    risk_rationale: str | None = None
    twitter_handle: str | None = None
    owner_name: str | None = None
    owner_link_url: str | None = None
    owner_link_text: str | None = None
    version_id: int = Field(alias="version_id")
    version_sid: str = Field(alias="version_sid")


class SymphonyVersionInfo(BaseModel):
    """Symphony version information."""

    version_id: str = Field(alias="version_id")
    created_at: str = Field(alias="created_at")


class TickersResponse(BaseModel):
    """Response for symphony tickers endpoint."""

    tickers: list[str]


class AISymphonyDescription(BaseModel):
    """AI-generated description for a symphony."""

    summary: str
    categories: str
    how_it_works: str = Field(alias="how_it_works")
    value_proposition: str = Field(alias="value_proposition")


class UserSymphony(BaseModel):
    """Symphony in user's library."""

    id: str | None = Field(alias="id", default=None)
    name: str | None = None
    description: str | None = None
    asset_classes: list[AssetClass] | None = Field(default_factory=list)
    rebalance_frequency: str | None = Field(alias="rebalance_frequency", default=None)
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)
    sparkgraph_gcs_url: str | None = Field(alias="sparkgraph_gcs_url", default=None)
    color: str | None = None
    num_node_asset: int | None = Field(alias="num_node_asset", default=None)
    num_node_filter: int | None = Field(alias="num_node_filter", default=None)
    num_node_group: int | None = Field(alias="num_node_group", default=None)
    num_node_if: int | None = Field(alias="num_node_if", default=None)
    num_node_if_child: int | None = Field(alias="num_node_if_child", default=None)
    num_node_wt_cash_equal: int | None = Field(alias="num_node_wt_cash_equal", default=None)
    num_node_wt_cash_specified: int | None = Field(alias="num_node_wt_cash_specified", default=None)
    num_node_wt_inverse_vol: int | None = Field(alias="num_node_wt_inverse_vol", default=None)
    oos_annualized_rate_of_return: float | None = Field(
        alias="oos_annualized_rate_of_return", default=None
    )
    oos_annualized_turnover: float | None = Field(alias="oos_annualized_turnover", default=None)
    oos_btcusd_alpha: float | None = Field(alias="oos_btcusd_alpha", default=None)
    oos_btcusd_annualized_rate_of_return: float | None = Field(
        alias="oos_btcusd_annualized_rate_of_return", default=None
    )
    oos_btcusd_beta: float | None = Field(alias="oos_btcusd_beta", default=None)
    oos_btcusd_calmar_ratio: float | None = Field(alias="oos_btcusd_calmar_ratio", default=None)
    oos_btcusd_cumulative_return: float | None = Field(
        alias="oos_btcusd_cumulative_return", default=None
    )
    oos_btcusd_max_drawdown: float | None = Field(alias="oos_btcusd_max_drawdown", default=None)
    oos_btcusd_pearson_r: float | None = Field(alias="oos_btcusd_pearson_r", default=None)
    oos_btcusd_r_square: float | None = Field(alias="oos_btcusd_r_square", default=None)
    oos_btcusd_sharpe_ratio: float | None = Field(alias="oos_btcusd_sharpe_ratio", default=None)
    oos_btcusd_standard_deviation: float | None = Field(
        alias="oos_btcusd_standard_deviation", default=None
    )
    oos_btcusd_trailing_one_month_return: float | None = Field(
        alias="oos_btcusd_trailing_one_month_return", default=None
    )
    oos_btcusd_trailing_one_year_return: float | None = Field(
        alias="oos_btcusd_trailing_one_year_return", default=None
    )
    oos_btcusd_trailing_three_month_return: float | None = Field(
        alias="oos_btcusd_trailing_three_month_return", default=None
    )
    oos_calmar_ratio: float | None = Field(alias="oos_calmar_ratio", default=None)
    oos_cumulative_return: float | None = Field(alias="oos_cumulative_return", default=None)
    oos_herfindahl_index: float | None = Field(alias="oos_herfindahl_index", default=None)
    oos_kurtosis: float | None = Field(alias="oos_kurtosis", default=None)
    oos_max_drawdown: float | None = Field(alias="oos_max_drawdown", default=None)
    oos_num_backtest_days: int | None = Field(alias="oos_num_backtest_days", default=None)
    oos_sharpe_ratio: float | None = Field(alias="oos_sharpe_ratio", default=None)
    oos_skewness: float | None = Field(alias="oos_skewness", default=None)
    oos_sortino_ratio: float | None = Field(alias="oos_sortino_ratio", default=None)
    oos_spy_alpha: float | None = Field(alias="oos_spy_alpha", default=None)
    oos_spy_annualized_rate_of_return: float | None = Field(
        alias="oos_spy_annualized_rate_of_return", default=None
    )
    oos_spy_beta: float | None = Field(alias="oos_spy_beta", default=None)
    oos_spy_calmar_ratio: float | None = Field(alias="oos_spy_calmar_ratio", default=None)
    oos_spy_cumulative_return: float | None = Field(alias="oos_spy_cumulative_return", default=None)
    oos_spy_max_drawdown: float | None = Field(alias="oos_spy_max_drawdown", default=None)
    oos_spy_pearson_r: float | None = Field(alias="oos_spy_pearson_r", default=None)
    oos_spy_r_square: float | None = Field(alias="oos_spy_r_square", default=None)
    oos_spy_sharpe_ratio: float | None = Field(alias="oos_spy_sharpe_ratio", default=None)
    oos_spy_standard_deviation: float | None = Field(
        alias="oos_spy_standard_deviation", default=None
    )
    oos_spy_trailing_one_month_return: float | None = Field(
        alias="oos_spy_trailing_one_month_return", default=None
    )
    oos_spy_trailing_one_year_return: float | None = Field(
        alias="oos_spy_trailing_one_year_return", default=None
    )
    oos_spy_trailing_three_month_return: float | None = Field(
        alias="oos_spy_trailing_three_month_return", default=None
    )
    oos_standard_deviation: float | None = Field(alias="oos_standard_deviation", default=None)
    oos_tail_ratio: float | None = Field(alias="oos_tail_ratio", default=None)
    oos_top_five_percent_day_contribution: float | None = Field(
        alias="oos_top_five_percent_day_contribution", default=None
    )
    oos_top_one_day_contribution: float | None = Field(
        alias="oos_top_one_day_contribution", default=None
    )
    oos_top_ten_percent_day_contribution: float | None = Field(
        alias="oos_top_ten_percent_day_contribution", default=None
    )
    oos_trailing_one_day_return: float | None = Field(
        alias="oos_trailing_one_day_return", default=None
    )
    oos_trailing_one_month_return: float | None = Field(
        alias="oos_trailing_one_month_return", default=None
    )
    oos_trailing_one_week_return: float | None = Field(
        alias="oos_trailing_one_week_return", default=None
    )
    oos_trailing_one_year_return: float | None = Field(
        alias="oos_trailing_one_year_return", default=None
    )
    oos_trailing_three_month_return: float | None = Field(
        alias="oos_trailing_three_month_return", default=None
    )
    oos_trailing_two_week_return: float | None = Field(
        alias="oos_trailing_two_week_return", default=None
    )
    oos_win_rate: float | None = Field(alias="oos_win_rate", default=None)
    ai_description: AISymphonyDescription | None = Field(alias="ai_description", default=None)


class DraftSymphony(BaseModel):
    """Draft symphony with statistics."""

    symphony_sid: str | None = Field(alias="symphony_sid", default=None)
    name: str | None = None
    description: str | None = None
    asset_classes: list[AssetClass] | None = Field(default_factory=list)
    rebalance_frequency: str | None = Field(alias="rebalance_frequency", default=None)
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)
    sparkgraph_gcs_url: str | None = Field(alias="sparkgraph_gcs_url", default=None)
    color: str | None = None
    num_node_asset: int | None = Field(alias="num_node_asset", default=None)
    num_node_filter: int | None = Field(alias="num_node_filter", default=None)
    num_node_group: int | None = Field(alias="num_node_group", default=None)
    num_node_if: int | None = Field(alias="num_node_if", default=None)
    num_node_if_child: int | None = Field(alias="num_node_if_child", default=None)
    num_node_wt_cash_equal: int | None = Field(alias="num_node_wt_cash_equal", default=None)
    num_node_wt_cash_specified: int | None = Field(alias="num_node_wt_cash_specified", default=None)
    num_node_wt_inverse_vol: int | None = Field(alias="num_node_wt_inverse_vol", default=None)
    oos_annualized_rate_of_return: float | None = Field(
        alias="oos_annualized_rate_of_return", default=None
    )
    oos_annualized_turnover: float | None = Field(alias="oos_annualized_turnover", default=None)
    oos_calmar_ratio: float | None = Field(alias="oos_calmar_ratio", default=None)
    oos_cumulative_return: float | None = Field(alias="oos_cumulative_return", default=None)
    oos_herfindahl_index: float | None = Field(alias="oos_herfindahl_index", default=None)
    oos_kurtosis: float | None = Field(alias="oos_kurtosis", default=None)
    oos_max_drawdown: float | None = Field(alias="oos_max_drawdown", default=None)
    oos_num_backtest_days: int | None = Field(alias="oos_num_backtest_days", default=None)
    oos_sharpe_ratio: float | None = Field(alias="oos_sharpe_ratio", default=None)
    oos_skewness: float | None = Field(alias="oos_skewness", default=None)
    oos_sortino_ratio: float | None = Field(alias="oos_sortino_ratio", default=None)
    oos_standard_deviation: float | None = Field(alias="oos_standard_deviation", default=None)
    oos_tail_ratio: float | None = Field(alias="oos_tail_ratio", default=None)
    oos_top_five_percent_day_contribution: float | None = Field(
        alias="oos_top_five_percent_day_contribution", default=None
    )
    oos_top_one_day_contribution: float | None = Field(
        alias="oos_top_one_day_contribution", default=None
    )
    oos_top_ten_percent_day_contribution: float | None = Field(
        alias="oos_top_ten_percent_day_contribution", default=None
    )
    oos_trailing_one_day_return: float | None = Field(
        alias="oos_trailing_one_day_return", default=None
    )
    oos_trailing_one_month_return: float | None = Field(
        alias="oos_trailing_one_month_return", default=None
    )
    oos_trailing_one_week_return: float | None = Field(
        alias="oos_trailing_one_week_return", default=None
    )
    oos_trailing_one_year_return: float | None = Field(
        alias="oos_trailing_one_year_return", default=None
    )
    oos_trailing_three_month_return: float | None = Field(
        alias="oos_trailing_three_month_return", default=None
    )
    oos_trailing_two_week_return: float | None = Field(
        alias="oos_trailing_two_week_return", default=None
    )
    oos_win_rate: float | None = Field(alias="oos_win_rate", default=None)
    ai_description: AISymphonyDescription | None = Field(alias="ai_description", default=None)


class WatchlistSymphony(BaseModel):
    """Symphony on user's watchlist."""

    id: str | None = Field(alias="id", default=None)
    name: str | None = None
    description: str | None = None
    asset_classes: list[AssetClass] | None = Field(default_factory=list)
    rebalance_frequency: str | None = Field(alias="rebalance_frequency", default=None)
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)
    sparkgraph_gcs_url: str | None = Field(alias="sparkgraph_gcs_url", default=None)
    color: str | None = None
    num_node_asset: int | None = Field(alias="num_node_asset", default=None)
    num_node_filter: int | None = Field(alias="num_node_filter", default=None)
    num_node_group: int | None = Field(alias="num_node_group", default=None)
    num_node_if: int | None = Field(alias="num_node_if", default=None)
    num_node_if_child: int | None = Field(alias="num_node_if_child", default=None)
    num_node_wt_cash_equal: int | None = Field(alias="num_node_wt_cash_equal", default=None)
    num_node_wt_cash_specified: int | None = Field(alias="num_node_wt_cash_specified", default=None)
    num_node_wt_inverse_vol: int | None = Field(alias="num_node_wt_inverse_vol", default=None)
    oos_annualized_rate_of_return: float | None = Field(
        alias="oos_annualized_rate_of_return", default=None
    )
    oos_annualized_turnover: float | None = Field(alias="oos_annualized_turnover", default=None)
    oos_calmar_ratio: float | None = Field(alias="oos_calmar_ratio", default=None)
    oos_cumulative_return: float | None = Field(alias="oos_cumulative_return", default=None)
    oos_herfindahl_index: float | None = Field(alias="oos_herfindahl_index", default=None)
    oos_max_drawdown: float | None = Field(alias="oos_max_drawdown", default=None)
    oos_sharpe_ratio: float | None = Field(alias="oos_sharpe_ratio", default=None)
    oos_sortino_ratio: float | None = Field(alias="oos_sortino_ratio", default=None)
    oos_standard_deviation: float | None = Field(alias="oos_standard_deviation", default=None)
    oos_tail_ratio: float | None = Field(alias="oos_tail_ratio", default=None)
    oos_trailing_one_month_return: float | None = Field(
        alias="oos_trailing_one_month_return", default=None
    )
    oos_trailing_one_year_return: float | None = Field(
        alias="oos_trailing_one_year_return", default=None
    )
    oos_trailing_three_month_return: float | None = Field(
        alias="oos_trailing_three_month_return", default=None
    )
    oos_win_rate: float | None = Field(alias="oos_win_rate", default=None)
    ai_description: AISymphonyDescription | None = Field(alias="ai_description", default=None)


class UserSymphoniesResponse(BaseModel):
    """Response for user's symphonies."""

    symphonies: list[UserSymphony]


class DraftSymphoniesResponse(BaseModel):
    """Response for user's draft symphonies."""

    symphonies: list[DraftSymphony]


class WatchlistResponse(BaseModel):
    """Response for watchlist."""

    symphonies: list[WatchlistSymphony] | None = Field(default_factory=list)
    watchlist: list[WatchlistSymphony] | None = Field(default_factory=list)

    def __init__(self, **data):
        if "symphonies" in data and "watchlist" not in data:
            data["watchlist"] = data.pop("symphonies")
        super().__init__(**data)


class SymphonyTickersResponse(BaseModel):
    """Response for symphony tickers."""

    tickers: list[str]


class WatchlistSymphonyItem(BaseModel):
    """Symphony item when adding to watchlist."""

    id: str | None = None
    asset_classes: list[AssetClass] | None = Field(default_factory=list)
    tickers: list[str] | None = Field(default_factory=list)
    name: str | None = None
    color: str | None = None
    description: str | None = None
    last_semantic_update_at: str | None = Field(alias="last_semantic_update_at", default=None)
    tags: list[str] | None = Field(default_factory=list)
    rebalance_frequency: str | None = Field(alias="rebalance_frequency", default=None)
    is_shared: bool | None = Field(alias="is_shared", default=None)
    rebalance_corridor_width: float | None = Field(alias="rebalance_corridor_width", default=None)
    annualized_rate_of_return: float | None = Field(alias="annualized_rate_of_return", default=None)
    simple_return: float | None = Field(alias="simple_return", default=None)
    sharpe_ratio: float | None = Field(alias="sharpe_ratio", default=None)
    calmar_ratio: float | None = Field(alias="calmar_ratio", default=None)
    standard_deviation: float | None = Field(alias="standard_deviation", default=None)
    trailing_one_month_return: float | None = Field(alias="trailing_one_month_return", default=None)
    trailing_three_month_return: float | None = Field(
        alias="trailing_three_month_return", default=None
    )
    max_drawdown: float | None = Field(alias="max_drawdown", default=None)
    last_backtest_holdings: dict[str, float] | None = Field(
        alias="last_backtest_holdings", default=None
    )
    last_backtest_value: float | None = Field(alias="last_backtest_value", default=None)
    owns_symphony: bool | None = Field(alias="owns_symphony", default=None)
    watched_since: str | None = Field(alias="watched_since", default=None)
    created_at: str | None = Field(alias="created_at", default=None)


class ModifySymphonyResponse(BaseModel):
    """Response for modify symphony operation."""

    symphony_id: str = Field(alias="symphony_id")
    version_id: str = Field(alias="version_id")


class FindAndReplaceOperation(BaseModel):
    """Find and replace operation for modifying symphonies."""

    op: str = Field(default="FIND_AND_REPLACE")
    old_ticker: str = Field(alias="old_ticker")
    new_ticker: str = Field(alias="new_ticker")


class CompressNestedIfsModification(BaseModel):
    """Compress nested ifs operation for modifying symphonies."""

    op: str = Field(default="COMPRESS_NESTED_IFS")
    node_id: str | None = Field(
        None,
        description=(
            "If provided, compress only the subtree rooted at this node. "
            "If omitted, compress the entire symphony."
        ),
    )


class BulkModifySymphoniesRequest(BaseModel):
    """Request for bulk modifying user symphonies."""

    op: str = Field(default="FIND_AND_REPLACE")
    user_id: str | None = Field(alias="user_id", default=None)
    old_ticker: str = Field(alias="old_ticker")
    new_ticker: str = Field(alias="new_ticker")


class UpdateSymphonyResponse(BaseModel):
    """Response for updating a symphony."""

    existing_version_id: str = Field(alias="existing_version_id")
    version_id: str = Field(alias="version_id")


class UpdateSymphonyNodesResponse(BaseModel):
    """Response for updating symphony nodes."""

    symphony_id: str = Field(alias="symphony_id")
    version_id: str = Field(alias="version_id")



"""Trading resource for trading-related endpoints."""

from ..models.trading import (
    CreateOrderRequest,
    CreateOrderResponse,
    ExercisePreviewResponse,
    ExerciseResponse,
    ModifyOrderRequest,
    OrderRequest,
    OrderRequestsResponse,
    OrderType,
    PositionIntent,
    TimeInForce,
    TradingPeriodResponse,
)


class Trading:
    """Resource for trading-related endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def get_trading_period(self) -> TradingPeriodResponse:
        """
        Get the trading period for the authenticated user.

        Returns
        -------
            Trading period information for CRYPTO, EQUITIES, and OPTIONS
        """
        response = self._client.get("/api/v1/trading-period")
        return TradingPeriodResponse.model_validate(response)

    def get_order_requests(
        self,
        account_id: str,
        limit: int | None = None,
        status: str | None = None,
    ) -> OrderRequestsResponse:
        """
        Get order requests for an account.

        Args:
            account_id: Unique identifier (UUID) of the account
            limit: Maximum number of results
            status: Filter by status (e.g., 'QUEUED,OPEN,IN_PROGRESS')

        Returns
        -------
            List of order requests
        """
        params = {}
        if limit is not None:
            params["limit"] = limit
        if status:
            params["status"] = status
        response = self._client.get(
            f"/api/v1/trading/accounts/{account_id}/order-requests",
            params=params if params else None,
        )
        return OrderRequestsResponse.model_validate(response)

    def get_order_request(
        self,
        account_id: str,
        order_request_id: str,
    ) -> OrderRequest:
        """
        Get a single order request.

        Args:
            account_id: Unique identifier (UUID) of the account
            order_request_id: Unique identifier for the order request

        Returns
        -------
            Details of the specified order request
        """
        response = self._client.get(
            f"/api/v1/trading/accounts/{account_id}/order-requests/{order_request_id}"
        )
        return OrderRequest.model_validate(response)

    def create_order_request(
        self,
        account_id: str,
        type: OrderType | str,
        symbol: str,
        time_in_force: TimeInForce | str,
        notional: float | None = None,
        quantity: float | None = None,
        position_intent: PositionIntent | str | None = None,
        limit_price: float | None = None,
        stop_price: float | None = None,
        client_order_id: str | None = None,
    ) -> CreateOrderResponse:
        """
        Create a new order request.

        Args:
            account_id: Unique identifier (UUID) of the account
            type: Order type (MARKET, LIMIT, STOP, STOP_LIMIT, TRAILING_STOP)
            symbol: Symbol to trade
            time_in_force: Time in force (GTC, DAY, IOC, FOK, OPG, CLS)
            notional: Notional amount (required if quantity not provided)
            quantity: Quantity (required if notional not provided)
            position_intent: Position intent for options (BUY_TO_OPEN, etc.)
            limit_price: Limit price for limit orders
            stop_price: Stop price for stop orders
            client_order_id: Optional client-generated order ID

        Returns
        -------
            Created order details
        """
        request = CreateOrderRequest(
            type=type if isinstance(type, OrderType) else OrderType(type),
            symbol=symbol,
            time_in_force=time_in_force
            if isinstance(time_in_force, TimeInForce)
            else TimeInForce(time_in_force),
            notional=notional,
            quantity=quantity,
            position_intent=position_intent
            if position_intent is None or isinstance(position_intent, PositionIntent)
            else PositionIntent(position_intent),
            limit_price=limit_price,
            stop_price=stop_price,
            client_order_id=client_order_id,
        )
        response = self._client.post(
            f"/api/v1/trading/accounts/{account_id}/order-requests",
            json=request.model_dump(exclude_none=True),
        )
        return CreateOrderResponse.model_validate(response)

    def delete_order_request(
        self,
        account_id: str,
        order_request_id: str,
    ) -> None:
        """
        Cancel an order request that has not executed yet.

        Args:
            account_id: Unique identifier (UUID) of the account
            order_request_id: Unique identifier for the order request
        """
        self._client.delete(
            f"/api/v1/trading/accounts/{account_id}/order-requests/{order_request_id}"
        )

    def modify_order_request(
        self,
        account_id: str,
        order_request_id: str,
        client_order_id: str | None = None,
        limit_price: float | None = None,
        quantity: float | None = None,
    ) -> None:
        """
        Modify an existing order request that has not executed yet.

        Args:
            account_id: Unique identifier (UUID) of the account
            order_request_id: Unique identifier for the order request
            client_order_id: ID for replacement order (idempotency key)
            limit_price: New limit price
            quantity: New quantity
        """
        request = ModifyOrderRequest(
            client_order_id=client_order_id,
            limit_price=limit_price,
            quantity=quantity,
        )
        self._client.patch(
            f"/api/v1/trading/accounts/{account_id}/order-requests/{order_request_id}",
            json=request.model_dump(exclude_none=True),
        )

    def exercise_option(
        self,
        account_id: str,
        symbol: str,
    ) -> ExerciseResponse:
        """
        Exercise an option contract position.

        Args:
            account_id: Unique identifier (UUID) of the account
            symbol: Option symbol (e.g., OPTIONS::AAPL1234567890PC20240119//USD)

        Returns
        -------
            Exercise result
        """
        response = self._client.post(
            f"/api/v1/trading/accounts/{account_id}/options/exercise",
            json={"symbol": symbol},
        )
        return ExerciseResponse.model_validate(response)

    def preview_exercise_option(
        self,
        account_id: str,
        symbol: str,
    ) -> ExercisePreviewResponse:
        """
        Preview exercise of an option contract position.

        Args:
            account_id: Unique identifier (UUID) of the account
            symbol: Option symbol (e.g., OPTIONS::AAPL1234567890PC20240119//USD)

        Returns
        -------
            Exercise preview
        """
        response = self._client.post(
            f"/api/v1/trading/accounts/{account_id}/options/exercise/preview",
            json={"symbol": symbol},
        )
        return ExercisePreviewResponse.model_validate(response)

    def preview_order_request(
        self,
        account_id: str,
        type: str,
        symbol: str,
        time_in_force: str,
        notional: float | None = None,
        quantity: float | None = None,
        position_intent: str | None = None,
        limit_price: float | None = None,
        stop_price: float | None = None,
    ) -> ExercisePreviewResponse:
        """
        Preview an order request before placing it.

        Args:
            account_id: Unique identifier (UUID) of the account
            type: Order type (MARKET, LIMIT, STOP, STOP_LIMIT, TRAILING_STOP)
            symbol: Symbol to trade
            time_in_force: Time in force (GTC, DAY, IOC, FOK, OPG, CLS)
            notional: Notional amount (required if quantity not provided)
            quantity: Quantity (required if notional not provided)
            position_intent: Position intent for options (BUY_TO_OPEN, etc.)
            limit_price: Limit price for limit orders
            stop_price: Stop price for stop orders

        Returns
        -------
            Order preview
        """
        request = CreateOrderRequest(
            type=type if isinstance(type, OrderType) else OrderType(type),
            symbol=symbol,
            time_in_force=time_in_force
            if isinstance(time_in_force, TimeInForce)
            else TimeInForce(time_in_force),
            notional=notional,
            quantity=quantity,
            position_intent=position_intent
            if position_intent is None or isinstance(position_intent, PositionIntent)
            else PositionIntent(position_intent),
            limit_price=limit_price,
            stop_price=stop_price,
            client_order_id=None,
        )
        response = self._client.post(
            f"/api/v1/trading/accounts/{account_id}/order-requests/preview",
            json=request.model_dump(exclude_none=True),
        )
        return ExercisePreviewResponse.model_validate(response)



"""Deploy models - response models for deploy endpoints."""

from pydantic import BaseModel, Field


class MarketHoursItem(BaseModel):
    """Market hours for a single day."""

    is_market_open: bool = Field(description="Whether the market is open on this date")
    nyse_market_date: str = Field(description="NYSE market date (YYYY-MM-DD)")
    market_open: str | None = Field(None, description="Market open time (ISO 8601)")
    market_close: str | None = Field(None, description="Market close time (ISO 8601)")


class MarketHoursResponse(BaseModel):
    """Response containing market hours schedule."""

    market_hours: list[MarketHoursItem] = Field(description="List of market hours for the week")


class DeploySymphony(BaseModel):
    """Symphony with pending invests."""

    symphony_id: str = Field(description="Unique identifier of the symphony")
    name: str = Field(description="Name of the symphony")
    asset_class: str = Field(description="Deprecated. See asset_classes.")
    asset_classes: list[str] = Field(description="Asset classes for the symphony")
    description: str = Field(description="Description of the symphony")
    color: str = Field(description="Color associated with the symphony")
    community_review_status: str | None = Field(None, description="Community review status")
    last_semantic_update_at: str = Field(description="Last semantic update time (ISO 8601)")
    rebalance_frequency: str = Field(description="Rebalance frequency")
    rebalance_corridor_width: float | None = Field(None, description="Rebalance corridor width")
    is_shared: bool = Field(description="Whether the symphony is shared")


class DeploySymphoniesResponse(BaseModel):
    """Response containing symphonies with pending invests."""

    symphonies: list[DeploySymphony] = Field(description="List of symphonies with pending invests")


class Deploy(BaseModel):
    """A deployment of a symphony to an account."""

    position_id: str | None = Field(None, description="Position ID")
    symphony_id: str | None = Field(None, description="Symphony ID")
    deploy_created_by: str | None = Field(None, description="User who created the deploy")
    deploy_id: str = Field(description="Unique identifier for the deployment")
    type: str = Field(description="Type of deployment")
    status: str = Field(description="Status of the deployment")
    cash_change: float | None = Field(None, description="Cash change")
    requested_cash_change: float | None = Field(None, description="Requested cash change")
    pending_buying_power: bool = Field(description="Whether buying power is pending")
    created_at: str = Field(description="When the deploy was created (ISO 8601)")


class DeploysResponse(BaseModel):
    """Response containing list of deploys."""

    deploys: list[Deploy] = Field(description="List of deploys")


class DeployActionResponse(BaseModel):
    """Response from a deploy action (invest, withdraw, liquidate, etc.)."""

    deploy_id: str = Field(description="Unique identifier for the deployment")
    deploy_time: str | None = Field(None, description="When the deploy is scheduled")
    deploy_on_market_open: bool | None = Field(
        None, description="Whether deploy executes on market open"
    )
    symphony_id: str | None = Field(None, description="Symphony ID")


__all__ = [
    "MarketHoursItem",
    "MarketHoursResponse",
    "DeploySymphony",
    "DeploySymphoniesResponse",
    "Deploy",
    "DeploysResponse",
    "DeployActionResponse",
]



"""Common statistics models shared across all API sections."""

from pydantic import BaseModel, Field


class RegressionMetrics(BaseModel):
    """Regression metrics for benchmark comparison (alpha, beta, etc.)."""

    alpha: float | None = None
    beta: float | None = None
    r_square: float | None = Field(None, alias="r_square")
    pearson_r: float | None = None


class BenchmarkStats(BaseModel):
    """Statistics for a benchmark comparison."""

    calmar_ratio: float | None = None
    cumulative_return: float | None = None
    max_: float | None = Field(None, alias="max")
    max_drawdown: float | None = None
    mean: float | None = None
    median: float | None = None
    min_: float | None = Field(None, alias="min")
    skewness: float | None = None
    kurtosis: float | None = None
    sharpe_ratio: float | None = None
    sortino_ratio: float | None = None
    size: int | None = None
    standard_deviation: float | None = None
    annualized_rate_of_return: float | None = None
    annualized_turnover: float | None = None
    trailing_one_day_return: float | None = None
    trailing_one_week_return: float | None = None
    trailing_two_week_return: float | None = None
    trailing_one_month_return: float | None = None
    trailing_three_month_return: float | None = None
    trailing_one_year_return: float | None = None
    top_one_day_contribution: float | None = None
    top_five_percent_day_contribution: float | None = None
    top_ten_percent_day_contribution: float | None = None
    herfindahl_index: float | None = None
    win_rate: float | None = None
    tail_ratio: float | None = None
    percent: RegressionMetrics | None = None
    log: RegressionMetrics | None = None

    class Config:
        """Pydantic model configuration."""

        populate_by_name = True


class Stats(BaseModel):
    """
    Performance statistics for portfolios, backtests, and symphonies.

    Contains all key performance metrics like Sharpe ratio, cumulative return,
    max drawdown, and various trailing returns.
    """

    calmar_ratio: float | None = None
    cumulative_return: float | None = None
    max_: float | None = Field(None, alias="max")
    max_drawdown: float | None = None
    mean: float | None = None
    median: float | None = None
    min_: float | None = Field(None, alias="min")
    skewness: float | None = None
    kurtosis: float | None = None
    sharpe_ratio: float | None = None
    sortino_ratio: float | None = None
    size: int | None = None
    standard_deviation: float | None = None
    annualized_rate_of_return: float | None = None
    annualized_turnover: float | None = None
    trailing_one_day_return: float | None = None
    trailing_one_week_return: float | None = None
    trailing_two_week_return: float | None = None
    trailing_one_month_return: float | None = None
    trailing_three_month_return: float | None = None
    trailing_one_year_return: float | None = None
    top_one_day_contribution: float | None = None
    top_five_percent_day_contribution: float | None = None
    top_ten_percent_day_contribution: float | None = None
    herfindahl_index: float | None = None
    win_rate: float | None = None
    tail_ratio: float | None = None
    benchmarks: dict[str, BenchmarkStats] | None = None

    class Config:
        """Pydantic model configuration."""

        populate_by_name = True

    def __repr__(self) -> str:
        """Return string representation of Stats."""
        parts = []
        if self.sharpe_ratio is not None:
            parts.append(f"sharpe={self.sharpe_ratio:.2f}")
        if self.cumulative_return is not None:
            parts.append(f"cumulative={self.cumulative_return:.2%}")
        if self.max_drawdown is not None:
            parts.append(f"drawdown={self.max_drawdown:.2%}")
        if self.annualized_rate_of_return is not None:
            parts.append(f"ann_return={self.annualized_rate_of_return:.2%}")
        return f"Stats({', '.join(parts)})"

    def __str__(self) -> str:
        """Return string representation of Stats."""
        return self.__repr__()
        return self.__repr__()



"""Dry run models."""

from enum import StrEnum

from pydantic import BaseModel, Field


class TradeSide(StrEnum):
    """Trade side."""

    BUY = "BUY"
    SELL = "SELL"


class RecommendedTrade(BaseModel):
    """A recommended trade in a dry run or preview."""

    model_config = {"populate_by_name": True}

    ticker: str
    notional: float
    quantity: float
    prev_value: float
    prev_weight: float
    next_weight: float


class PreviewRecommendedTrade(BaseModel):
    """A recommended trade in a trade preview."""

    model_config = {"populate_by_name": True}

    at: str
    average_price: float
    cash_change: float
    share_change: float
    symbol: str
    name: str | None = None
    side: TradeSide
    type: str
    prev_value: float
    prev_weight: float
    next_weight: float


class DryRunResult(BaseModel):
    """Dry run result for a single symphony."""

    model_config = {"populate_by_name": True}

    rebalanced: bool
    next_rebalance_after: str
    recommended_trades: list[RecommendedTrade]
    queued_cash_change: float
    symphony_value: float
    symphony_name: str
    next_rebalance_date: str | None = None


class TradePreviewResult(BaseModel):
    """Trade preview result for a symphony."""

    model_config = {"populate_by_name": True}

    rebalanced: bool
    next_rebalance_after: str
    recommended_trades: list[PreviewRecommendedTrade]
    queued_cash_change: float
    rebalance_frequency_override: bool
    rebalance_request_uuid: str | None = None
    adjusted_for_dtbp: bool
    symphony_value: float
    symphony_name: str


class AccountDryRunResult(BaseModel):
    """Dry run result for an account."""

    model_config = {"populate_by_name": True}

    broker_account_uuid: str
    account_type: str
    account_name: str
    broker: str
    dry_run_result: dict
    dry_run_missing_symphonies: dict | None = None
    dry_run_total_symphonies: int


class DryRunRequest(BaseModel):
    """Request body for dry run endpoint."""

    model_config = {"populate_by_name": True}

    account_uuids: list[str] | None = None
    send_segment_event: bool = False


class TradePreviewRequest(BaseModel):
    """Request body for trade preview endpoint."""

    model_config = {"populate_by_name": True}

    amount: float | None = None
    broker_account_uuid: str | None = None


class AccountSource(StrEnum):
    """Account source for dry run."""

    QUEUED = "queued"
    DEPLOYABLE = "deployable"
    ALL_TRADABLE = "all_tradable"


class CreateDryRunRequest(BaseModel):
    """Request body for rebalance dry run endpoint."""

    account_source: AccountSource = Field(
        default=AccountSource.QUEUED, description="Account source to include in dry run"
    )
    dry_run_time: str | None = Field(None, description="Time to run the dry run (ISO 8601 format)")


class DryRunEnqueueResponse(BaseModel):
    """Response from enqueuing a dry run."""

    num_accounts: int = Field(description="Number of accounts included in dry run")
    dry_run_time: str = Field(description="Time the dry run was triggered")



"""Search resource for publicly shared symphonies."""

from typing import Any

from ..models.search import SearchSymphonyResult


class Search:
    """Resource for searching publicly shared symphonies."""

    def __init__(self, http_client):
        self.http_client = http_client

    def search_symphonies(
        self,
        where: list[Any] | None = None,
        order_by: list[list[str]] | None = None,
        offset: int = 0,
    ) -> list[SearchSymphonyResult]:
        """
        Search our database of publicly shared symphonies.

        Args:
            where: Filter conditions for the search. Uses HoneySQL (Clojure) syntax.
                   Format: ["and", [condition1], [condition2], ...]
                   Example: ["and", [">", "oos_num_backtest_days", 180]]
            order_by: Sorting criteria as list of [field, direction] pairs.
                      For example: [["oos_sharpe_ratio", "desc"]]
            offset: Pagination offset for results

        Returns
        -------
            List of matching symphonies with performance statistics
        """
        if where is None:
            where = []
        request: dict[str, Any] = {"offset": offset}
        if where is not None:
            request["where"] = where
        if order_by is not None:
            request["order_by"] = order_by

        response = self.http_client.post(
            "/api/v1/public/search/symphonies",
            json=request,
        )

        return [SearchSymphonyResult.model_validate(r) for r in response]

    def search_symphonies_v2(
        self,
        filter: str | None = None,
        order_by: list[list[str]] | None = None,
        offset: int = 0,
    ) -> list[SearchSymphonyResult]:
        """
        Search publicly shared symphonies using CEL filters (V2).

        Args:
            filter: CEL filter expression for advanced filtering.
                    Example: "oos_sharpe_ratio > 1.5"
            order_by: Sorting criteria as list of [field, direction] pairs.
                      For example: [["oos_sharpe_ratio", "desc"]]
            offset: Pagination offset for results

        Returns
        -------
            List of matching symphonies with performance statistics
        """
        request: dict[str, Any] = {"offset": offset}
        if filter is not None:
            request["filter"] = filter
        if order_by is not None:
            request["order_by"] = order_by

        response = self.http_client.post(
            "/api/v1/public/search/symphonies-v2",
            json=request,
        )
        return [SearchSymphonyResult.model_validate(r) for r in response]



"""Dry run resource for dry-run endpoints."""

from ..models.dry_run import (
    AccountDryRunResult,
    DryRunRequest,
    TradePreviewRequest,
    TradePreviewResult,
)


class DryRun:
    """Resource for dry-run endpoints."""

    def __init__(self, http_client):
        self._client = http_client

    def create_dry_run(
        self,
        send_segment_event: bool = False,
        account_uuids: list[str] | None = None,
    ) -> list[AccountDryRunResult]:
        """
        Dry run rebalances for a specific user.

        Args:
            send_segment_event: Whether to send segment events
            account_uuids: Optional list of specific account UUIDs

        Returns
        -------
            List of dry run results for each account
        """
        request = DryRunRequest(
            account_uuids=account_uuids,
            send_segment_event=send_segment_event,
        )
        response = self._client.post(
            "/api/v1/dry-run",
            json=request.model_dump(exclude_none=True),
        )
        return [AccountDryRunResult.model_validate(r) for r in response]

    def create_trade_preview(
        self,
        symphony_id: str,
        amount: float | None = None,
        broker_account_uuid: str | None = None,
    ) -> TradePreviewResult:
        """
        Generate trade preview for a single symphony.

        Args:
            symphony_id: Unique identifier for the Symphony
            amount: Optional amount to preview
            broker_account_uuid: Optional specific account UUID

        Returns
        -------
            Trade preview result
        """
        request = TradePreviewRequest(
            amount=amount,
            broker_account_uuid=broker_account_uuid,
        )
        response = self._client.post(
            f"/api/v1/dry-run/trade-preview/{symphony_id}",
            json=request.model_dump(exclude_none=True),
        )
        return TradePreviewResult.model_validate(response)



"""Backtest request models (input models)."""

from enum import StrEnum
from typing import Any

from pydantic import BaseModel, Field

from ..common.symphony import SymphonyDefinition


class BacktestVersion(StrEnum):
    """Backtest version enum."""

    V1 = "v1"
    V2 = "v2"


class Broker(StrEnum):
    """Broker enum."""

    ALPACA_OAUTH = "ALPACA_OAUTH"
    ALPACA_WHITE_LABEL = "ALPACA_WHITE_LABEL"
    APEX_LEGACY = "APEX_LEGACY"
    ALPACA = "alpaca"
    APEX = "apex"


class ApplySubscription(StrEnum):
    """Subscription type enum."""

    NONE = "none"
    MONTHLY = "monthly"
    YEARLY = "yearly"


class BacktestParams(BaseModel):
    """
    Common parameters for all backtest requests.

    These parameters control how the backtest is run including
    capital amount, fees, slippage, and date range.

    All parameters have sensible defaults so you can run a backtest
    with minimal configuration.
    """

    model_config = {"populate_by_name": True}

    capital: float = Field(default=10000.0, description="Initial capital for the backtest")
    abbreviate_days: int | None = Field(
        None, description="Number of days to abbreviate the backtest (for testing)"
    )
    apply_reg_fee: bool = Field(
        default=True, description="Whether to apply regulatory fees (SEC fees)"
    )
    apply_taf_fee: bool = Field(
        default=True, description="Whether to apply TAF (Trading Activity Fee)"
    )
    apply_subscription: ApplySubscription = Field(
        default=ApplySubscription.NONE,
        description="Composer subscription level to simulate (affects fees)",
    )
    backtest_version: BacktestVersion = Field(
        default=BacktestVersion.V2, description="Backtest engine version to use"
    )
    slippage_percent: float = Field(
        default=0.0001, description="Slippage assumption as decimal (0.0001 = 0.01%)"
    )
    spread_markup: float = Field(
        default=0.0, description="Bid-ask spread markup as decimal (0.001 = 0.1%)"
    )
    start_date: str | None = Field(
        None,
        description="Backtest start date (YYYY-MM-DD). Defaults to earliest available data.",
    )
    end_date: str | None = Field(
        None,
        description="Backtest end date (YYYY-MM-DD). Defaults to latest available data.",
    )
    broker: Broker = Field(
        default=Broker.ALPACA_WHITE_LABEL,
        description="Broker to simulate for fee calculations",
    )
    benchmark_symphonies: list[str] | None = Field(
        None, description="List of symphony IDs to use as benchmarks"
    )
    benchmark_tickers: list[str] | None = Field(
        None,
        description="List of ticker symbols to use as benchmarks (e.g., ['SPY', 'QQQ'])",
    )
    sparkgraph_color: str | None = Field(None, description="Custom color for performance chart")


class BacktestRequest(BacktestParams):
    """
    Full backtest request with symphony definition.

    Use this when backtesting a new symphony by providing the full
    symphony definition in the request body.
    """

    symphony: SymphonyDefinition = Field(description="Symphony definition to backtest")


class BacktestExistingSymphonyRequest(BacktestParams):
    """
    Backtest request for an existing saved symphony.

    Use this when backtesting a symphony that has already been saved.
    The symphony_id is provided in the URL path, not the request body.
    """

    pass


class Quote(BaseModel):
    """Quote for a single ticker in rebalance."""

    ticker: str
    trading_halted: bool = False
    open: float
    close: float | None = None
    low: float
    high: float
    volume: int
    bid: dict[str, Any] | None = None
    ask: dict[str, Any] | None = None
    last: dict[str, Any] | None = None
    source: str | None = None
    timestamp: str | None = None


class SymphonyRebalanceState(BaseModel):
    """State of a single symphony for rebalance."""

    last_rebalanced_on: str | None = None
    cash: float
    unsettled_cash: float = 0.0
    shares: dict[str, float]


class RebalanceRequest(BaseModel):
    """
    Request body for the rebalance endpoint.

    Used to run a rebalance for specified symphonies given a starting state.
    """

    dry_run: bool = False
    broker: Broker = Broker.ALPACA_WHITE_LABEL
    adjust_for_dtbp: bool = False
    disable_fractional_trading: bool = False
    fractionability: dict[str, bool] | None = None
    end_date: str | None = None
    quotes: dict[str, Quote] | None = None
    symphonies: dict[str, SymphonyRebalanceState]



"""Accounts models - request and response models for accounts endpoints."""

from pydantic import BaseModel, Field

from .responses import (
    Account,
    AccountInfo,
    AccountsListResponse,
    AccountType,
    Address,
    AssetClass,
    AvailableAccountType,
    BuyingPower,
    Contact,
    DirectTradableAssetClasses,
    FootprintPlaybook,
    Holding,
    Identity,
    IncomeRange,
    InvestorDocument,
    InvestorDocumentCategory,
    InvestorProfile,
    OptionsDetails,
    SupportedRegionsResponse,
    SupportedRegionStates,
    Trade,
    TradeHistoryItem,
    TradeVolumeResponse,
    TrustedContact,
)


class ApiKey(BaseModel):
    """API key information."""

    id: str = Field(description="API key ID")


class ApiKeyWithSecret(BaseModel):
    """API key with secret (only returned on creation)."""

    id: str = Field(description="API key ID")
    secret: str = Field(description="API key secret")


class ApiKeysResponse(BaseModel):
    """Response containing list of API keys."""

    api_keys: list[ApiKey] = Field(description="List of API keys")


class CreateApiKeyResponse(BaseModel):
    """Response from creating an API key."""

    api_key: ApiKeyWithSecret = Field(description="Created API key")


__all__ = [
    "AssetClass",
    "DirectTradableAssetClasses",
    "Account",
    "OptionsDetails",
    "Holding",
    "InvestorDocument",
    "InvestorDocumentCategory",
    "AccountsListResponse",
    "AccountType",
    "AvailableAccountType",
    "SupportedRegionsResponse",
    "SupportedRegionStates",
    "FootprintPlaybook",
    "Trade",
    "TradeVolumeResponse",
    "TradeHistoryItem",
    "Identity",
    "Address",
    "Contact",
    "IncomeRange",
    "InvestorProfile",
    "TrustedContact",
    "AccountInfo",
    "BuyingPower",
    "ApiKey",
    "ApiKeysResponse",
    "CreateApiKeyResponse",
    "ApiKeyWithSecret",
]
